/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.backend.BackendFactory;
import org.eclipse.samples.organizer.core.Activator;
import org.eclipse.samples.organizer.events.ContentChangeEvent;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.listeners.ContentChangeListener;

public class OrganizerManager {
	private static final String PREFERRED_TASK_BACKEND_NAME = "task_backend";
	
	private static final String PREFERRED_MAIL_BACKEND_NAME = "mail_backend";
	
	private static final String PREFERRED_CONTACT_BACKEND_NAME = "contact_backend";
	
	private static final String PREFERRED_ACCOUNT_BACKEND_NAME = "account_backend";

	public static OrganizerManager instance = new OrganizerManager();

	public List<BackendFactory> backendFactories;

	private ListenerList contentChangeListenerList = new ListenerList();

	/**
	 * This method returns a list of all backends known to the receiver. It
	 * builds this list by consulting all known backend factories which can each
	 * contribute zero or more backends. A factory that is based on a database,
	 * might return multiple backends, each of which represents data stored in a
	 * different database. A factory based on the Eclipse resources API, might
	 * return a separate backend for each project that contains organizer data.
	 * 
	 * @return A list of {@link Backend} instances.
	 */
	public List<Backend> getBackends() {
		List<Backend> backends = new ArrayList<Backend>();
		for (BackendFactory factory : backendFactories) {
			backends.addAll(factory.getBackends());
		}
		return backends;
	}

	/**
	 * This method return the collection of all known backend factories. A
	 * factory is responsible for providing a collection of zero or more
	 * backends that contain organizer data. Factories are found through the
	 * <code>org.eclipse.samples.organizer.core.factory</code> extension-point. Note
	 * that once this method is called, the plug-ins containing the factory
	 * extensions are loaded; an indirection was not considered since the
	 * results of this method are immediately interrogated for their
	 * corresponding backends (there is no point in avoiding the load, since the
	 * next step will force a load anyway).
	 * 
	 * @return A {@link List} of {@link BackendFactory} instances.
	 */
	private List<BackendFactory> findBackendFactories() {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry
				.getExtensionPoint("org.eclipse.samples.organizer.core.factory");
		IExtension[] extensions = point.getExtensions();

		List<BackendFactory> factories = new ArrayList<BackendFactory>();
		for (IExtension element : extensions) {
			addBackendFactories(factories, element.getConfigurationElements());
		}

		return factories;
	}

	/**
	 * This method builds instances of the backend factory from the
	 * configuration elements. A factory must implement the
	 * <code>BackendFactory</code> interface. This method is resilient to
	 * errors in the specification of factories; it will log and ignore errors
	 * as they are encountered.
	 * 
	 * @param factories
	 *            The list of {@link BackendFactory} instances to which newly
	 *            found and created factories will be added.
	 * @param configurationElements
	 *            An array of {@link IConfigurationElement} instances containing
	 *            the specification of the factories.
	 */
	private void addBackendFactories(List<BackendFactory> factories,
			IConfigurationElement[] configurationElements) {
		for (IConfigurationElement element : configurationElements) {
			if ("class".equals(element.getName())) {
				try {
					Object object = element.createExecutableExtension("name");

					if (object instanceof BackendFactory) {
						try {
							BackendFactory factory = (BackendFactory) object;
							factory.initialize(this);
							factories.add(factory);
						} catch (Exception e) {
							Activator.logError(e);
						}
					} else {
						Activator.logError(String.valueOf(element.getValue())
								+ " must subclass "
								+ BackendFactory.class.getName());
					}
				} catch (CoreException e) {
					Activator.logError(e.getStatus());
				}
			}
		}
	}

	@SuppressWarnings("unchecked")
	public Object[] find(Class type) throws BackendException {
		return find(new Filter(type));
	}
	
	@SuppressWarnings("unchecked")
	public Object[] find(Filter filter) throws BackendException {
		final List objects = new ArrayList();
		find(filter, new Visitor() {
			public void visit(Object object) {
				objects.add(object);
			}
		});
		return objects.toArray(new Object[objects.size()]);
	}
	/**
	 * This method invokes the given visitor with the objects
	 * that match the provided filter. 
	 * @throws BackendException 
	 */
	public void find(Filter filter, Visitor visitor) throws BackendException {
		for (Backend backend : getBackends()) {
			backend.find(filter, visitor);		
		}
	}
	
	/**
	 * This method is used to add new tasks to a backend.
	 * 
	 * @param backend
	 *            The {@link Backend} that will receive the new tasks.
	 * @param objects
	 *            The list of objects to add.
	 * @throws Exception
	 */
	public void add(Backend backend, List<?> objects) throws BackendException {
		backend.add(objects);
		fireContentChangeEvent(new ContentChangeEvent());
	}
	
	
	public void delete(Backend backend, List<?> objects) throws BackendException {
		backend.delete(objects);
		fireContentChangeEvent(new ContentChangeEvent());
	}

	/**
	 * This method returns the name of the preferred backend for tasks. This
	 * information, persisted in the plug-in preferences, is typically (although
	 * not necessarily) the name of the last backend targetted for addition of
	 * organizer tasks.
	 * 
	 * @return The name of the preferred backend or an empty string if none is
	 *         specified.
	 */
	public String getPreferredTaskBackendName() {
		return getPluginPreferences().getString(PREFERRED_TASK_BACKEND_NAME);
	}
	/**
	 * This method returns the name of the preferred backend for mails. This
	 * information, persisted in the plug-in preferences, is typically (although
	 * not necessarily) the name of the last backend targetted for addition of
	 * organizer mails.
	 * 
	 * @return The name of the preferred backend or an empty string if none is
	 *         specified.
	 */
	public String getPreferredMailBackendName() {
		return getPluginPreferences().getString(PREFERRED_MAIL_BACKEND_NAME);
	}
	
	public String getPreferredContactBackendName() {
		return getPluginPreferences().getString(PREFERRED_CONTACT_BACKEND_NAME);
	}
	public String getPreferredAccountBackendName() {
		return getPluginPreferences().getString(PREFERRED_ACCOUNT_BACKEND_NAME);
	}
	/**
	 * This method sets the name of the preferred backend for tasks. This
	 * information is stored in the plug-ins preferences.
	 * 
	 * @param name The name of the preferred backend.
	 */
	public void setPreferredTaskBackendName(String name) {
		if (name == null) 
			getPluginPreferences().setToDefault(PREFERRED_TASK_BACKEND_NAME);
		else 
			getPluginPreferences().setValue(PREFERRED_TASK_BACKEND_NAME, name);
	}
	/**
	 * This method sets the name of the preferred backend for mails. This
	 * information is stored in the plug-ins preferences.
	 * 
	 * @param name The name of the preferred backend.
	 */
	public void setPreferredMailBackendName(String name) {
		if (name == null) 
			getPluginPreferences().setToDefault(PREFERRED_MAIL_BACKEND_NAME);
		else 
			getPluginPreferences().setValue(PREFERRED_MAIL_BACKEND_NAME, name);
	}
	public void setPreferredContactBackendName(String name) {
		if (name == null) 
			getPluginPreferences().setToDefault(PREFERRED_CONTACT_BACKEND_NAME);
		else 
			getPluginPreferences().setValue(PREFERRED_CONTACT_BACKEND_NAME, name);
	}
	
	public void setPreferredAccountBackendName(String name) {
		if (name == null) 
			getPluginPreferences().setToDefault(PREFERRED_ACCOUNT_BACKEND_NAME);
		else 
			getPluginPreferences().setValue(PREFERRED_ACCOUNT_BACKEND_NAME, name);
	}

	private Preferences getPluginPreferences() {
		return Activator.getDefault().getPluginPreferences();
	}

	public void initialize() {
		backendFactories = findBackendFactories();
	}

	public void shutdown() {
		for (BackendFactory factory : backendFactories) {
			try {
				factory.shutdown();
			} catch (Exception e) {
				Activator.logError(e);
			}
		}
	}

	public Backend getPreferredTaskBackend() {
		String backendName = getPreferredTaskBackendName();
		if (backendName == null)
			return null;
		for (Backend backend : getBackends()) {
			if (backend.getName().equals(backendName))
				return backend;
		}
		return null;
	}

	public Backend getPreferredMailBackend() {
		String backendName = getPreferredMailBackendName();
		if (backendName == null)
			return null;
		for (Backend backend : getBackends()) {
			if (backend.getName().equals(backendName))
				return backend;
		}
		return null;
	}
	public Backend getPreferredContactBackend() {
		//String backendName = getPreferredContactBackendName();
		String backendName = null;
		if (backendName == null)
			return null;
		for (Backend backend : getBackends()) {
			if (backend.getName().equals(backendName))
				return backend;
		}
		return null;
	}
	public Backend getPreferredAccountBackend() {
		String backendName = getPreferredAccountBackendName();
		if (backendName == null)
			return null;
		for (Backend backend : getBackends()) {
			if (backend.getName().equals(backendName))
				return backend;
		}
		return null;
	}
	
	public void setPreferredTaskBackend(Backend backend) {
		assert backend != null;
		setPreferredTaskBackendName(backend.getName());
	}
	
	public void setPreferredMailBackend(Backend backend) {
		assert backend != null;
		setPreferredMailBackendName(backend.getName());
	}
	public void setPreferredContactBackend(Backend backend) {
		assert backend != null;
		setPreferredContactBackendName(backend.getName());
	}
	public void setPreferredAccountBackend(Backend backend) {
		assert backend != null;
		setPreferredAccountBackendName(backend.getName());
	}


	public void delete(List<?> objects) {
		
	}

	public void addContentChangeListener(ContentChangeListener contentChangeListener) {
		contentChangeListenerList.add(contentChangeListener);
	}
	
	public void removeContentChangeListener(ContentChangeListener contentChangeListener) {
		contentChangeListenerList.remove(contentChangeListener);
	}
	
	private void fireContentChangeEvent(ContentChangeEvent event) {
		for (Object object : contentChangeListenerList.getListeners()) {
			ContentChangeListener listener = (ContentChangeListener)object;
			listener.contentChanged(event);
		}
	}

	public int count(Filter filter) throws BackendException {
		int count = 0;
		for (Backend backend : getBackends()) {
			count += backend.count(filter);
		}
		return count;
	}
}
