package org.eclipse.samples.organizer.derby;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.ref.WeakReference;
import java.util.ConcurrentModificationException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.backend.Backend;

public abstract class AbstractBackendWithLocalCache implements Backend {
	protected OrganizerManager manager;
	private Map<CacheKey, WeakReference<?>> cache = new HashMap<CacheKey, WeakReference<?>>();


	private PropertyChangeListener propertyChangeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent event) {
			update(event.getSource());			
		}
	};
	
	public AbstractBackendWithLocalCache() {
		super();
	}
	
	protected abstract void update(Object object);

	/**
	 * The cache uses weak references so that, should the cache
	 * be the only reference to the object, that object will be subject
	 * to garbage collection. In that event, the cache will be left
	 * with an empty slot (the key and the weak reference objects
	 * remain).
	 */
	protected Object findInCache(Class<?> type, String id) {
		WeakReference<?> ref = cache.get(new CacheKey(type, id));
		if (ref == null)
			return null;
		return ref.get();
	}

	protected void cache(Class<?> type, String id, Object object) {
		hookPropertyChangeListener(object);
		cache.put(new CacheKey(object.getClass(), id), new WeakReference<Object>(object));
	}

	public Object getTasksCacheSize() {
		cleanCache();
		return cache.size();
	}

	protected void cleanCache() {
		try {
			for (CacheKey key : cache.keySet()) {
				WeakReference<?> ref = cache.get(key);
				if (ref.get() == null) {
					cache.remove(key);
				}
			}
		} catch (ConcurrentModificationException e) {
			// If we get a conncurrent modification exception,
			// we just stop and wait for our next opportunity
			// to try. We'll get it eventually...
		}
	}

	void hookPropertyChangeListener(Object object) {
		hookPropertyChangeListener(object, propertyChangeListener);
	}

	protected abstract void hookPropertyChangeListener(Object object, PropertyChangeListener propertyChangeListener);
}

class CacheKey {
	private final Class<?> type;
	private final String id;

	public CacheKey(Class<?> type, String id) {
		this.type = type;
		this.id = id;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((type == null) ? 0 : type.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final CacheKey other = (CacheKey) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (type == null) {
			if (other.type != null)
				return false;
		} else if (!type.equals(other.type))
			return false;
		return true;
	}

}
