package org.eclipse.samples.organizer.derby;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.Date;

import org.eclipse.samples.organizer.OrganizerManager;

public abstract class AbstractDatabaseBackend extends
		AbstractBackendWithLocalCache {

	protected Database database;

	public AbstractDatabaseBackend(Database database) {
		this.database = database;
	}
	
	protected abstract void createTables(Connection connection) throws SQLException;

	protected abstract boolean isDatabaseConfigured(Connection connection) throws SQLException;
	
	public void initialize(OrganizerManager manager) throws Exception {
		Database database = getDatabase();
		Connection connection = null;
		try {
			connection = database.getConnection();
			if (isDatabaseConfigured(connection)) createTables(connection);
		} finally {
			close(connection);
		}
	}
	
	public void shutdown() {
	}

	protected Database getDatabase() {
		return database;
	}
	
	protected String getClob(ResultSet results, int index) throws SQLException,
			IOException {
				Clob clob = results.getClob(index);
				if (clob == null) return null;
				Reader reader = clob.getCharacterStream();
				try {
					return extractValue(reader);
				} finally {
					reader.close();
				}
			}

	private String extractValue(Reader reader) throws IOException {
		StringBuilder builder = new StringBuilder();
		char[] buffer = new char[1024];
		int count = reader.read(buffer);
		while (count > 0) {
			builder.append(buffer, 0, count);
			count = reader.read(buffer);
		}
		return builder.toString();
	}

	protected Date getDate(ResultSet results, int index) throws SQLException {
		java.sql.Date date = results.getDate(index);
		if (date == null) return null;
		return new Date(date.getTime());
	}

	protected Connection getConnection() throws SQLException {
		return database.getConnection();
	}

	/**
	 * This method sets a date value into the statement. This method
	 * exists to handle the case when the value is null (which is not
	 * handled by the Date.setDate() method.
	 */
	protected void setDate(PreparedStatement statement, int index, Date value)
			throws SQLException {
				if (value == null)
					statement.setNull(index, Types.DATE);
				else
					statement.setDate(index, new java.sql.Date(value.getTime()));
			}

	/**
	 * This method sets a string value into the statement. This method
	 * exists to handle the case when the value is null (which is not
	 * handled by the Date.setString() method.
	 */
	protected void setString(PreparedStatement statement, int index, String value)
			throws SQLException {
				if (value == null)
					statement.setNull(index, Types.VARCHAR);
				else
					statement.setString(index, value);
			}

	/**
	 * This method sets a string value into the statement. This method
	 * exists to handle the case when the value is null (which is not
	 * handled by the Date.setCharacterStream() method.
	 */
	protected void setClob(PreparedStatement statement, int index, String value)
			throws SQLException {
				if (value == null)
					statement.setNull(index, Types.CLOB);
				else
					statement.setCharacterStream(index, new StringReader(value), value.length());
			}

	protected void close(Statement statement) {
		if (statement == null)
			return;
		try {
			statement.close();
		} catch (Exception e) {
			Activator.logError(e);
		}
	}

	protected void close(Connection connection) {
		if (connection == null)
			return;
		try {
			connection.close();
		} catch (Exception e) {
			Activator.logError(e);
		}
	}


}