/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.derby;

import java.beans.PropertyChangeListener;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import org.eclipse.samples.organizer.Filter;
import org.eclipse.samples.organizer.Visitor;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.mails.filters.AccountFilter;
import org.eclipse.samples.organizer.mails.filters.MailFilter;
import org.eclipse.samples.organizer.mails.filters.MailFilterByFolderID;
import org.eclipse.samples.organizer.mails.filters.MailFilterByUnread;
import org.eclipse.samples.organizer.main.MailAccount;
import org.eclipse.samples.organizer.models.Mail;
// Changed by Ha Li, to separate different backend, such as task,mail,contact backend.
public class AccountDerbyBackend extends AbstractDatabaseBackend implements Backend {
	 private static final String CREATE_ACCOUNT_TABLE_STATEMENT = "create table organizer.accounts (id int generated always as identity primary key, account_name varchar(512), email_address varchar(512), logon_name varchar(512),email_password varchar(512), incomingserver_type varchar(512), incomingserver_address varchar(512), incomingserver_port int, outgoingserver_type varchar(512), outgoingserver_address varchar(512), outgoingserver_port int,delete_getting int,use_ssl int,root_folderID int)";

	private static final String SELECT_ACCOUNTS_STATEMENT = "select account_name ,email_address ,logon_name ,email_password , incomingserver_type  ,incomingserver_address ,incomingserver_port ,outgoingserver_type ,outgoingserver_address ,outgoingserver_port , delete_getting ,use_ssl ,root_folderID,id from organizer.mails where 1=1";
	private static final String ACCOUNTS_TABLE_EXISTS_STATEMENT = "select * from organizer.accounts";
	private static final String INSERT_ACCOUNT_STATEMENT = "insert into organizer.accounts (account_name ,email_address ,logon_name ,email_password , incomingserver_type  ,incomingserver_address ,incomingserver_port ,outgoingserver_type ,outgoingserver_address ,outgoingserver_port , delete_getting ,use_ssl ,root_folderID,id) values (?,?,?,?,?,?,?,?,?,?,?,?,?, DEFAULT)";
	private static final String DELETE_ACCOUNT_STATEMENT = "delete from organizer.accounts where id =?";
	private static final String UPDATE_ACCOUNT_STATEMENT = "update organizer.accounts set account_name=? ,email_address=? ,logon_name=? ,email_password=? , incomingserver_type=?  ,incomingserver_address=? ,incomingserver_port=? ,outgoingserver_type=? ,outgoingserver_address=? ,outgoingserver_port=? , delete_getting=? ,use_ssl=? ,root_folderID=? where id=?";
	private static final String COUNT_ACCOUNTS_STATEMENT = "select count(*) from organizer.accounts where 1=1";

	// column value for mails
	private static final int COLUMN_ACCOUNT_NAME  = 1;
	private static final int COLUMN_EMAIL_ADDRESS = 2;
	private static final int COLUMN_LOGON_NAME = 3;
	private static final int COLUMN_EMAIL_PASSWORD  = 4;
	private static final int COLUMN_INCOMINGSERVER_TYPE  = 5;
	private static final int COLUMN_INCOMINGSERVER_ADDRESS  = 6;
	private static final int COLUMN_INCOMINGSERVER_PORT = 7;
	private static final int COLUMN_OUTGOINGSERVER_TYPE = 8;
	private static final int COLUMN_OUTGOINGSERVER_ADDRESS  = 9;
	private static final int COLUMN_OUTGOINGSERVER_PORT = 10;
	private static final int COLUMN_DELETE_GETTING = 11;
	private static final int COLUMN_USE_SSL = 12;
	private static final int COLUMN_ROOT_FOLDERID = 13;
	private static final int COLUMN_ID = 14;
	
	public AccountDerbyBackend(Database database) {
		super(database);
	}

	public String getName() {
		return "Local Mail Organizer Database";
	}
		
	@Override
	protected boolean isDatabaseConfigured(Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = connection.prepareStatement(ACCOUNTS_TABLE_EXISTS_STATEMENT);
			statement.setMaxRows(1);
			statement.execute();
			return true;
		} catch (Exception e) {
			// TODO If it is not a "table not found" exception, rethrow.
			return false;
		} finally {
			close(statement);
		}
	}
	
	@Override
	protected void createTables(Connection connection) throws SQLException {
		Statement statement = null;
		try {
			statement = connection.createStatement();
			statement.execute(CREATE_ACCOUNT_TABLE_STATEMENT);
		} finally {
			close(statement);
		}
	}
	
	public void add(List<?> objects) throws BackendException {
		Connection connection = null;
		try {
			connection = getConnection();
			add(objects, connection);
		} catch (SQLException e) {
			throw new BackendException(e);
		} finally {
			close(connection);
		}
	}

	public void find(Filter filter, Visitor visitor) {
		if(true)
			return;
		if(!(filter instanceof AccountFilter))
			return ;
		Connection connection = null;
		try {
			connection = getConnection();
			find(filter, visitor, connection);	
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			close(connection);
		}
	}

	private void find(Filter filter, Visitor visitor, Connection connection) throws Exception {
		cleanCache();
		if (filter.includesType(MailAccount.class)) {
			findAccounts(filter,visitor, connection);
		}
	}
	
	private void findAccounts(Filter filter, Visitor visitor, Connection connection) throws Exception {
		PreparedStatement statement = null;
		try {
			statement = generateStatement(filter, connection, SELECT_ACCOUNTS_STATEMENT);
			ResultSet results = statement.executeQuery();
			while (results.next()) {
				MailAccount account = buildAccount(results);
				visitor.visit(account);
			}
		} finally {
			close(statement);
		}
	}

	// TODO Need to actually consider filter.
	private PreparedStatement generateStatement(Filter filter, Connection connection, String baseQuery) throws SQLException {
		String query = buildQuery(filter, baseQuery);
		PreparedStatement statement = connection.prepareStatement(query);
		return statement;
	}
	
	private String buildQuery(Filter filter,String baseQuery) {
		    StringBuilder query = null;
			query =new StringBuilder(baseQuery);
			if(MailFilterByUnread.instance.isUnreadCheck()){
				MailFilterByUnread.instance.setUnreadCheck(false);
				query.append(" and have_readed =0");
				return query.toString();
			}
			MailFilterByFolderID mailFilter =(MailFilterByFolderID)filter;
			query.append(" and folderID =" + mailFilter.getFolderID());
			//mailFilter.removeFilter();
			
		
		return query.toString();
	}

	private MailAccount buildAccount(ResultSet results) throws Exception {
		final String id = String.valueOf(results.getInt(COLUMN_ID));
		MailAccount account = findCachedOrCreateNewAccount(id);
		account.setAccountName(results.getString(COLUMN_ACCOUNT_NAME));
		account.setEmail(results.getString(COLUMN_EMAIL_ADDRESS));
		account.setLogin(results.getString(COLUMN_LOGON_NAME));
		account.setPassword(results.getString(COLUMN_EMAIL_PASSWORD));
		account.setIncomingtype(results.getString(COLUMN_INCOMINGSERVER_TYPE));
		account.setIncomingAddress(results.getString(COLUMN_INCOMINGSERVER_TYPE));
		account.setIncomingPort(results.getInt(COLUMN_INCOMINGSERVER_PORT));
		account.setOutgoingtype(results.getString(COLUMN_OUTGOINGSERVER_TYPE));
		account.setOutgoingAddress(results.getString(COLUMN_OUTGOINGSERVER_TYPE));
		account.setOutgoingPort(results.getInt(COLUMN_OUTGOINGSERVER_PORT));
		account.setDeletedFromServer(results.getInt(COLUMN_DELETE_GETTING)==0?false:true);
		account.setUseSSL(results.getInt(COLUMN_USE_SSL)==0?false:true);
		account.setRootFolderID(results.getInt(COLUMN_ROOT_FOLDERID));
		return account;
	}
	
	/**
	 * Mails are cached by the backend in order to preserve identity.
	 * The cache uses weak references so that, should the cache
	 * be the only reference to the task, that task will be subject
	 * to garbage collection. In that event, the cache will be left
	 * with an empty slot (the key and the weak reference objects
	 * remain).
	 */
	private MailAccount findCachedOrCreateNewAccount(String id) {
		MailAccount account = (MailAccount)findInCache(MailAccount.class, id);
		if (account == null) {
			account = new MailAccount();
			account.setAccountID(Integer.parseInt(id));
			cache(Mail.class, id, account);
		}
		return account;
	}


	private void add(List<?> objects, Connection connection)
			throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = connection.prepareStatement(INSERT_ACCOUNT_STATEMENT,
					Statement.RETURN_GENERATED_KEYS);
			for (Object object : objects) {

				MailAccount account = (MailAccount) object;

				
				setString(statement, COLUMN_ACCOUNT_NAME,account.getAccountName());
				setString(statement, COLUMN_EMAIL_ADDRESS,account.getEmail());
				setString(statement, COLUMN_LOGON_NAME,account.getLogin());
				setString(statement, COLUMN_EMAIL_PASSWORD,account.getPassword());
				setString(statement, COLUMN_INCOMINGSERVER_TYPE,account.getIncomingtype());
				setString(statement, COLUMN_INCOMINGSERVER_ADDRESS,account.getIncomingAddress());
				statement.setInt(COLUMN_INCOMINGSERVER_PORT, account.getIncomingPort());
				setString(statement, COLUMN_OUTGOINGSERVER_TYPE,account.getOutgoingtype());
				setString(statement, COLUMN_OUTGOINGSERVER_ADDRESS,account.getOutgoingAddress());
				statement.setInt(COLUMN_OUTGOINGSERVER_PORT, account.getOutgoingPort());
				
				statement.setInt(COLUMN_DELETE_GETTING, account.isDeletedFromServer()== true ?0 :1);
				statement.setInt(COLUMN_USE_SSL, account.isUseSSL()== true ?0 :1);
				statement.setInt(COLUMN_ROOT_FOLDERID, account.getRootFolderID());
				

				// Get the key for the added item.
				ResultSet keys = statement.getGeneratedKeys();
				keys.next();
				account.setAccountID(keys.getInt(1));
				String id = String.valueOf(account.getAccountID());
				cache(MailAccount.class, id, account);
			}
		} finally {
			close(statement);
		}
	}

	@Override
	protected void update(Object object) {
		if (object instanceof MailAccount) updateMail((MailAccount)object);
	}
	
	private void updateMail(MailAccount account) {
		Connection connection = null;
		PreparedStatement statement = null;
		try {
			connection = getConnection();
			statement = connection.prepareStatement(UPDATE_ACCOUNT_STATEMENT);
			
			setString(statement, COLUMN_ACCOUNT_NAME,account.getAccountName());
			setString(statement, COLUMN_EMAIL_ADDRESS,account.getEmail());
			setString(statement, COLUMN_LOGON_NAME,account.getLogin());
			setString(statement, COLUMN_EMAIL_PASSWORD,account.getPassword());
			setString(statement, COLUMN_INCOMINGSERVER_TYPE,account.getIncomingtype());
			setString(statement, COLUMN_INCOMINGSERVER_ADDRESS,account.getIncomingAddress());
			statement.setInt(COLUMN_INCOMINGSERVER_PORT, account.getIncomingPort());
			setString(statement, COLUMN_OUTGOINGSERVER_TYPE,account.getOutgoingtype());
			setString(statement, COLUMN_OUTGOINGSERVER_ADDRESS,account.getOutgoingAddress());
			statement.setInt(COLUMN_OUTGOINGSERVER_PORT, account.getOutgoingPort());
			
			statement.setInt(COLUMN_DELETE_GETTING, account.isDeletedFromServer()== true ?0 :1);
			statement.setInt(COLUMN_USE_SSL, account.isUseSSL()== true ?0 :1);
			statement.setInt(COLUMN_ROOT_FOLDERID, account.getRootFolderID());
			statement.setInt(COLUMN_ID,account.getAccountID());
			statement.execute();
		} catch (Exception e) {
			Activator.logError(e);
		} finally {
			close(statement);
			close(connection);
		}
		
	}

	public int count(Filter filter) throws BackendException {
		if(!(filter instanceof MailFilter))
			return 0;
		Connection connection = null;
		try {
			connection = getConnection();
			return count(filter, connection);
		} catch (SQLException e) {
			throw new BackendException(e);

		} finally {
			close(connection);
		}
	}
	
	private int count(Filter filter, Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = generateStatement(filter, connection, COUNT_ACCOUNTS_STATEMENT);
			ResultSet results = statement.executeQuery();
			if (!results.next()) return 0;
			return results.getInt(1);
		} finally {
			close(statement);
		}
	}

	public void delete(List<?> objects) throws BackendException {
		PreparedStatement statement = null;
		Connection connection = null;
		try {
			connection = getConnection();
			statement = connection.prepareStatement(DELETE_ACCOUNT_STATEMENT);
			for (Object object : objects) {
				MailAccount account = (MailAccount) object;
				statement.setInt(1, account.getAccountID());
				statement.execute();
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			close(statement);
			close(connection);
		}
	}

	@Override
	protected void hookPropertyChangeListener(Object object, PropertyChangeListener propertyChangeListener) {
		if (object instanceof Mail) {
			((Mail)object).addPropertyChangeListener(propertyChangeListener);
		}		
	}

}


