/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.derby;

import java.beans.PropertyChangeListener;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import org.eclipse.samples.organizer.Filter;
import org.eclipse.samples.organizer.Visitor;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.contacts.filters.ContactFilter;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.models.Contact;
import org.eclipse.samples.organizer.models.Task;
 
// Changed by Ha Li, to separate different backend, such as task,mail,contact backend.
public class ContactDerbyBackend extends AbstractDatabaseBackend implements Backend {
	private static final String SELECT_CONTACTS_STATEMENT = "select contact_name, primary_email, phone_num, id from organizer.contacts where 1=1";
	private static final String CONTACTS_TABLE_EXISTS_STATEMENT = "select * from organizer.contacts";
	private static final String INSERT_CONTACT_STATEMENT = "insert into organizer.contacts (contact_name, primary_email, phone_num, id) values (?, ?, ?, DEFAULT)";
	private static final String CREATE_CONTACTS_TABLE_STATEMENT = "create table organizer.contacts (id int generated always as identity primary key, contact_name varchar(512), primary_email varchar(512), phone_num int)";
	private static final String COUNT_CONTACTS_STATEMENT = "select count(*) from organizer.contacts where 1=1";

//	private static final String DELETE_MAILS_TABLE_STATEMENT = "drop table organizer.mails";
	private static final String DELETE_CONTACTS_TABLE_STATEMENT = "drop table organizer.contacts";
	// column value for mails
	private static final int CONTACT_COLUMN_NAME = 1;
	private static final int CONTACT_COLUMN_EMAILADDRESS = 2;
	private static final int CONTACT_COLUMN_PHONENUM = 3;
	private static final int CONTACT_COLUMN_ID = 4;
	
	public ContactDerbyBackend(Database database) {
		super(database);
	}

	public String getName() {
		return "Local Account Organizer Database";
	}
		
	@Override
	protected boolean isDatabaseConfigured(Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = connection.prepareStatement(CONTACTS_TABLE_EXISTS_STATEMENT);
			statement.setMaxRows(1);
			statement.execute();
			return true;
		} catch (Exception e) {
			// TODO If it is not a "table not found" exception, rethrow.
			return false;
		} finally {
			close(statement);
		}
	}
	
	@Override
	protected void createTables(Connection connection) throws SQLException {
		Statement statement = null;
		try {
			statement = connection.createStatement();
			statement.execute(CREATE_CONTACTS_TABLE_STATEMENT);
		} finally {
			close(statement);
		}
	}
	
	public void add(List<?> objects) throws BackendException {
		Connection connection = null;
		try {
			connection = getConnection();
			add(objects, connection);
		} catch (SQLException e) {
			throw new BackendException(e);
		} finally {
			close(connection);
		}
	}

	public void find(Filter filter, Visitor visitor) {
		if(!(filter instanceof ContactFilter))
			return;
		Connection connection = null;
		try {
			connection = getConnection();
			find(filter, visitor, connection);	
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			close(connection);
		}
	}

	private void find(Filter filter, Visitor visitor, Connection connection) throws Exception {
		cleanCache();
		if (filter.includesType(Contact.class)) {
			findContacts(filter,visitor, connection);
		}
	}
	
	private void findContacts(Filter filter, Visitor visitor, Connection connection) throws Exception {
		PreparedStatement statement = null;
		try {
			statement = generateStatement(filter, connection,SELECT_CONTACTS_STATEMENT);
			ResultSet results = statement.executeQuery();
			while (results.next()) {
				Contact contact = buildContact(results);
				visitor.visit(contact);
			}
		} finally {
			close(statement);
		}
	}

	// TODO Need to actually consider filter.
	private PreparedStatement generateStatement(Filter filter, Connection connection, String baseQuery) throws SQLException {
		String query = buildQuery(filter, baseQuery);
		PreparedStatement statement = connection.prepareStatement(query);
		return statement;
	}
	
	private String buildQuery(Filter filter, String baseQuery) {
		StringBuilder query = new StringBuilder(baseQuery);
		ContactFilter contactFilter = (ContactFilter) filter;
		if (contactFilter.getNameStart() == '*') {

		} else {
			char charlittle = contactFilter.getNameStart();
			char charbig = Character.isLowerCase(charlittle)? Character.toUpperCase(charlittle): Character.toLowerCase(charlittle);
			
			query.append(" and contact_name like '"
					+ charlittle + "%' or contact_name like '" 
					+ charbig + "%'");
			query.append(" or primary_email like '"
					+ charlittle + "%' or primary_email like '" 
					+ charbig + "%'");
		}
		return query.toString();
	}
	


	private Contact buildContact(ResultSet results) throws Exception {
		final String id = String.valueOf(results.getInt(CONTACT_COLUMN_ID));
		Contact contact = findCachedOrCreateNewContact(id);
		contact.setFullName(results.getString(CONTACT_COLUMN_NAME));
		contact.setPrimaryMailAddress(results.getString(CONTACT_COLUMN_EMAILADDRESS));
		contact.setMobileNum(results.getInt(CONTACT_COLUMN_PHONENUM));
		
				
		return contact;
	}
	
	/**
	 * Tasks are cached by the backend in order to preserve identity.
	 * The cache uses weak references so that, should the cache
	 * be the only reference to the contact, that contact will be subject
	 * to garbage collection. In that event, the cache will be left
	 * with an empty slot (the key and the weak reference objects
	 * remain).
	 */
	private Contact findCachedOrCreateNewContact(String id) {
		Contact contact = (Contact)findInCache(Contact.class, id);
		if (contact == null) {
			contact = new Contact();
			contact.setId(Integer.parseInt(id));
			cache(Contact.class, id, contact);
		}
		return contact;
	}


	private void add(List<?> objects, Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = connection.prepareStatement(INSERT_CONTACT_STATEMENT, Statement.RETURN_GENERATED_KEYS);
			for (Object object : objects) {
					Contact contact = (Contact)object; 
					setString(statement, CONTACT_COLUMN_NAME, contact.getFullName());
					setString(statement, CONTACT_COLUMN_EMAILADDRESS, contact.getPrimaryMailAddress());
					statement.setInt(CONTACT_COLUMN_PHONENUM , contact.getMobileNum());	
					statement.execute();
					// Get the key for the added item.
					ResultSet keys = statement.getGeneratedKeys();
					keys.next();
					contact.setId(keys.getInt(1));
					String id = String.valueOf(contact.getId());
					cache(Contact.class, id, contact);
			}
		} 
		finally {
			close(statement);
		}
	}

	@Override
	protected void update(Object object) {
		
	}


	public int count(Filter filter) throws BackendException {
		if(!(filter instanceof ContactFilter))
			return 0;
		Connection connection = null;
		try {
			connection = getConnection();
			return count(filter, connection);
		} catch (SQLException e) {
			throw new BackendException(e);

		} finally {
			close(connection);
		}
	}
	
	private int count(Filter filter, Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = generateStatement(filter, connection, COUNT_CONTACTS_STATEMENT);
			ResultSet results = statement.executeQuery();
			if (!results.next()) return 0;
			return results.getInt(1);
		} finally {
			close(statement);
		}
	}

	public void delete(List<?> objects) throws BackendException {
		// TODO Auto-generated method stub
		
	}

	@Override
	protected void hookPropertyChangeListener(Object object, PropertyChangeListener propertyChangeListener) {
		if (object instanceof Task) {
			((Task)object).addPropertyChangeListener(propertyChangeListener);
		}		
	}

}
