/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.derby;

import java.beans.PropertyChangeListener;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import org.eclipse.samples.organizer.Filter;
import org.eclipse.samples.organizer.Visitor;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.main.MailFolder;
import org.eclipse.samples.organizer.models.Task;
import org.eclipse.samples.organizer.tasks.filters.TaskFilter;
 
// Changed by Ha Li, to separate different backend, such as task,mail,contact backend.
public class FolderDerbyBackend extends AbstractDatabaseBackend implements Backend {
	private static final String FOLDERS_TABLE_EXISTS_STATEMENT = "select * from organizer.mailfolders";
	private static final String CREATE_FOLDERS_TABLE_STATEMENT = "create table organizer.mailfolders (id int generated always as identity primary key, folder_name varchar(512),is_account int, account_id int,parentfolder_id int, folder_path varchar(512), is_systemfolder int)";
	
	private static final String SELECT_FOLDERS_STATEMENT = "select folder_name , is_account, account_id, parentfolder_id, folder_path, is_systemfolder, id from organizer.mailfolders where 1=1";
	private static final String INSERT_FOLDERS_STATEMENT = "insert into organizer.tasks (older_name , is_account, account_id, parentfolder_id, folder_path, is_systemfolder, id) values (?, ?, ?, ?, ?, ?, DEFAULT)";
	private static final String UPDATE_FOLDER_STATEMENT = "update organizer.tasks set older_name=?, is_account=?, account_id=?, parentfolder_id=?, folder_path=?, is_systemfolder=? where id=?";
	private static final String COUNT_FOLDERS_STATEMENT = "select count(*) from organizer.mailfolders where 1=1";
	
	private static final int COLUMN_FOLDER_NAME = 1;
	private static final int COLUMN_IS_ACCOUNT = 2;
	private static final int COLUMN_ACCOUNT_ID = 3;
	private static final int COLUMN_PARENTFOLDER_ID = 4;
	private static final int COLUMN_FOLDER_PATH = 5;
	private static final int COLUMN_IS_SYSTEMFOLDER = 6;
	private static final int COLUMN_ID = 7;
	
	public FolderDerbyBackend(Database database) {
		super(database);
	}

	public String getName() {
		return "Local Mail Folder Organizer Database";
	}
		
	@Override
	protected boolean isDatabaseConfigured(Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = connection.prepareStatement(FOLDERS_TABLE_EXISTS_STATEMENT);
			statement.setMaxRows(1);
			statement.execute();
			return true;
		} catch (Exception e) {
			// TODO If it is not a "table not found" exception, rethrow.
			return false;
		} finally {
			close(statement);
		}
	}
	
	@Override
	protected void createTables(Connection connection) throws SQLException {
		Statement statement = null;
		try {
			statement = connection.createStatement();
			statement.execute(CREATE_FOLDERS_TABLE_STATEMENT);
		} finally {
			close(statement);
		}
	}
	
	public void add(List<?> objects) throws BackendException {
		Connection connection = null;
		try {
			connection = getConnection();
			add(objects, connection);
		} catch (SQLException e) {
			throw new BackendException(e);
		} finally {
			close(connection);
		}
	}

	public void find(Filter filter, Visitor visitor) {
	/*	if(!(filter instanceof AccountFilter))
			return;*/
		Connection connection = null;
		try {
			connection = getConnection();
			find(filter, visitor, connection);	
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			close(connection);
		}
	}

	private void find(Filter filter, Visitor visitor, Connection connection) throws Exception {
		cleanCache();
		if (filter.includesType(Task.class)) {
			findTasks(filter,visitor, connection);
		}
	}
	
	private void findTasks(Filter filter, Visitor visitor, Connection connection) throws Exception {
		PreparedStatement statement = null;
		try {
			statement = generateStatement(filter, connection, SELECT_FOLDERS_STATEMENT);
			ResultSet results = statement.executeQuery();
			while (results.next()) {
				MailFolder folder = buildFolder(results);
				visitor.visit(folder);
			}
		} finally {
			close(statement);
		}
	}

	// TODO Need to actually consider filter.
	private PreparedStatement generateStatement(Filter filter, Connection connection, String baseQuery) throws SQLException {
		String query = buildQuery(filter, baseQuery);
		PreparedStatement statement = connection.prepareStatement(query);
		return statement;
	}
	
	private String buildQuery(Filter filter, String baseQuery) {
		StringBuilder query = new StringBuilder(baseQuery);
		if (filter instanceof TaskFilter) {
			TaskFilter taskFilter = (TaskFilter)filter;
			if (taskFilter.isHideCompleteTasks()) {
				query.append(" and complete='n'");
			}
		}
		return query.toString();
	}

	private MailFolder buildFolder(ResultSet results) throws Exception {
		final String id = String.valueOf(results.getInt(COLUMN_ID));
		MailFolder folder = findCachedOrCreateNewFolder(id);
		
		folder.setFolderName(results.getString(COLUMN_FOLDER_NAME));
		folder.setIsAccount(results.getInt(COLUMN_IS_ACCOUNT)==0 ? false :true);
		folder.setAccountID(results.getInt(COLUMN_ACCOUNT_ID));
		folder.setParentFolderID(results.getInt(COLUMN_PARENTFOLDER_ID));
		folder.setFolderFilePath(results.getString(COLUMN_FOLDER_PATH));
		folder.setSystemfoldersymbol(results.getInt(COLUMN_IS_SYSTEMFOLDER)==0 ? false :true);
				
		return folder;
	}
	
	/**
	 * Tasks are cached by the backend in order to preserve identity.
	 * The cache uses weak references so that, should the cache
	 * be the only reference to the task, that task will be subject
	 * to garbage collection. In that event, the cache will be left
	 * with an empty slot (the key and the weak reference objects
	 * remain).
	 */
	private MailFolder findCachedOrCreateNewFolder(String id) {
		MailFolder folder = (MailFolder)findInCache(MailFolder.class, id);
		if (folder == null) {
			folder = new MailFolder();
			folder.setFolderID(Integer.parseInt(id));
			cache(MailFolder.class, id, folder);
		}
		return folder;
	}


	private void add(List<?> objects, Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = connection.prepareStatement(INSERT_FOLDERS_STATEMENT, Statement.RETURN_GENERATED_KEYS);
			for (Object object : objects) {
				if (object instanceof MailFolder) {
					MailFolder folder = (MailFolder)object; 
					setString(statement, COLUMN_FOLDER_NAME, folder.getFolderName());
					statement.setInt(COLUMN_IS_ACCOUNT, folder.isAccount()==true ?1:0);
					statement.setInt(COLUMN_ACCOUNT_ID, folder.getAccountID());
					statement.setInt(COLUMN_PARENTFOLDER_ID, folder.getParentFolderID());
					setString(statement, COLUMN_FOLDER_PATH, folder.getFolderFilePath());
					statement.setInt(COLUMN_IS_SYSTEMFOLDER, folder.isSystemfoldersymbol()==true ?1:0);
					statement.execute();
				
					// Get the key for the added item.
					ResultSet keys = statement.getGeneratedKeys();
					keys.next();
					int id = keys.getInt(1);
					folder.setFolderID(id);
					cache(MailFolder.class, String.valueOf(id), folder);
				} else {
					// TODO Throw an UnsupportedObjectException or something.
				}
			}
		} finally {
			close(statement);
		}
	}

	@Override
	protected void update(Object object) {
		if (object instanceof MailFolder) updateFolder((MailFolder)object);
	}
	
	private void updateFolder(MailFolder folder) {
		Connection connection = null;
		PreparedStatement statement = null;
		try {
			connection = getConnection();
			statement = connection.prepareStatement(UPDATE_FOLDER_STATEMENT);
			setString(statement, COLUMN_FOLDER_NAME, folder.getFolderName());
			statement.setInt(COLUMN_IS_ACCOUNT, folder.isAccount()==true ?1:0);
			statement.setInt(COLUMN_ACCOUNT_ID, folder.getAccountID());
			statement.setInt(COLUMN_PARENTFOLDER_ID, folder.getParentFolderID());
			setString(statement, COLUMN_FOLDER_PATH, folder.getFolderFilePath());
			statement.setInt(COLUMN_IS_SYSTEMFOLDER, folder.isSystemfoldersymbol()==true ?1:0);
			statement.setInt(COLUMN_ID, folder.getFolderID());
			statement.execute();
		} catch (Exception e) {
			Activator.logError(e);
		} finally {
			close(statement);
			close(connection);
		}
	}

	public int count(Filter filter) throws BackendException {
	/*	if(!(filter instanceof AccountFilter))
			return 0;*/
		Connection connection = null;
		try {
			connection = getConnection();
			return count(filter, connection);
		} catch (SQLException e) {
			throw new BackendException(e);

		} finally {
			close(connection);
		}
	}
	
	private int count(Filter filter, Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = generateStatement(filter, connection, COUNT_FOLDERS_STATEMENT);
			ResultSet results = statement.executeQuery();
			if (!results.next()) return 0;
			return results.getInt(1);
		} finally {
			close(statement);
		}
	}

	public void delete(List<?> objects) throws BackendException {
		// TODO Auto-generated method stub
		
	}

	@Override
	protected void hookPropertyChangeListener(Object object, PropertyChangeListener propertyChangeListener) {
		if (object instanceof Task) {
			((Task)object).addPropertyChangeListener(propertyChangeListener);
		}		
	}

}
