/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.derby;

import java.beans.PropertyChangeListener;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import org.eclipse.samples.organizer.Filter;
import org.eclipse.samples.organizer.Visitor;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.mails.filters.MailFilter;
import org.eclipse.samples.organizer.mails.filters.MailFilterByFolderID;
import org.eclipse.samples.organizer.mails.filters.MailFilterByUnread;
import org.eclipse.samples.organizer.models.Mail;
 
// Changed by Ha Li, to separate different backend, such as task,mail,contact backend.
public class MailDerbyBackend extends AbstractDatabaseBackend implements Backend {
	private static final String SELECT_MAILS_STATEMENT = "select datafile_path,have_readed,accout_id,send_date, sender_name ,receiver_name,cc_name,bcc_name,subject,attachment, priority,mail_flag,mail_format,mail_tag,folderid,mail_size,mail_uid,id from organizer.mails where 1=1";
	private static final String MAILS_TABLE_EXISTS_STATEMENT = "select * from organizer.mails";
	private static final String INSERT_MAIL_STATEMENT = "insert into organizer.mails (datafile_path,have_readed,accout_id,send_date, sender_name ,receiver_name,cc_name,bcc_name,subject,attachment, priority,mail_flag,mail_format,mail_tag,folderid,mail_size,mail_uid,id) values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?, DEFAULT)";
	private static final String DELETE_MAIL_STATEMENT = "delete from organizer.mails where id =?";
	private static final String UPDATE_MAIL_STATEMENT = "update organizer.mails set datafile_path=?, have_readed=?,accout_id=?,send_date=?, sender_name=? ,receiver_name=?,cc_name=?,bcc_name=?,subject=?,attachment=?, priority=?,mail_flag=?,mail_format=?,mail_tag=?,folderid=?,mail_size=?,mail_uid=? where id=?";
	private static final String CREATE_MAILS_TABLE_STATEMENT = "create table organizer.mails (id int generated always as identity primary key ,datafile_path varchar(512),have_readed int,accout_id int,send_date varchar(512),sender_name varchar(512),receiver_name varchar(512),cc_name varchar(512),bcc_name varchar(512),subject varchar(512),attachment int, priority int,mail_flag int,mail_format int,mail_tag int,folderid int,mail_size int,mail_uid varchar(512))";
	private static final String COUNT_MAILS_STATEMENT = "select count(*) from organizer.mails where 1=1";
     private static final String DELETE_MAIL_TABLE_STATEMENT = "drop table organizer.mails";
	// column value for mails
	private static final int MAIL_COLUMN_MAILFILEPATH  = 1;
	private static final int MAIL_COLUMN_HAVE_READ = 2;
	private static final int MAIL_COLUMN_ACCOUT_ID = 3;
	private static final int MAIL_COLUMN_SEND_DATE  = 4;
	private static final int MAIL_COLUMN_SENDER_NAME = 5;
	private static final int MAIL_COLUMN_RECEIVER_NAME = 6;
	private static final int MAIL_COLUMN_CC_NAME = 7;
	private static final int MAIL_COLUMN_BCC_NAME = 8;
	private static final int MAIL_COLUMN_SUBJECT = 9;
	private static final int MAIL_COLUMN_ATTACHMENT = 10;
	private static final int MAIL_COLUMN_PRIORITY = 11;
	private static final int MAIL_COLUMN_FLAG = 12;
	private static final int MAIL_COLUMN_FORMAT = 13;
	private static final int MAIL_COLUMN_TAG = 14;
	private static final int MAIL_COLUMN_FOLDERID = 15;
	private static final int MAIL_COLUMN_MESSAGE_SIZE = 16;
	private static final int MAIL_COLUMN_MESSAGE_UID = 17;
	private static final int MAIL_COLUMN_ID = 18;
	
	public MailDerbyBackend(Database database) {
		super(database);
	}

	public String getName() {
		return "Local Mail Organizer Database";
	}
		
	@Override
	protected boolean isDatabaseConfigured(Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = connection.prepareStatement(MAILS_TABLE_EXISTS_STATEMENT);
			statement.setMaxRows(1);
			statement.execute();
			return true;
		} catch (Exception e) {
			// TODO If it is not a "table not found" exception, rethrow.
			return false;
		} finally {
			close(statement);
		}
	}
	
	@Override
	protected void createTables(Connection connection) throws SQLException {
		Statement statement = null;
		try {
			statement = connection.createStatement();
		//	statement.execute(CREATE_MAILS_TABLE_STATEMENT);
			statement.execute(DELETE_MAIL_TABLE_STATEMENT);
		} finally {
			close(statement);
		}
	}
	
	public void add(List<?> objects) throws BackendException {
		Connection connection = null;
		try {
			connection = getConnection();
			add(objects, connection);
		} catch (SQLException e) {
			throw new BackendException(e);
		} finally {
			close(connection);
		}
	}

	public void find(Filter filter, Visitor visitor) {
		if(!(filter instanceof MailFilter))
			return;
		Connection connection = null;
		try {
			connection = getConnection();
			find(filter, visitor, connection);	
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			close(connection);
		}
	}

	private void find(Filter filter, Visitor visitor, Connection connection) throws Exception {
		cleanCache();
		if (filter.includesType(Mail.class)) {
			findMails(filter,visitor, connection);
		}
	}
	
	private void findMails(Filter filter, Visitor visitor, Connection connection) throws Exception {
		PreparedStatement statement = null;
		try {
			statement = generateStatement(filter, connection, SELECT_MAILS_STATEMENT);
			ResultSet results = statement.executeQuery();
			while (results.next()) {
				Mail mail = buildMail(results);
				visitor.visit(mail);
			}
		} finally {
			close(statement);
		}
	}

	// TODO Need to actually consider filter.
	private PreparedStatement generateStatement(Filter filter, Connection connection, String baseQuery) throws SQLException {
		String query = buildQuery(filter, baseQuery);
		PreparedStatement statement = connection.prepareStatement(query);
		return statement;
	}
	
	private String buildQuery(Filter filter,String baseQuery) {
		    StringBuilder query = null;
			query =new StringBuilder(baseQuery);
			if(MailFilterByUnread.instance.isUnreadCheck()){
				MailFilterByUnread.instance.setUnreadCheck(false);
				query.append(" and have_readed =0");
				return query.toString();
			}
			MailFilterByFolderID mailFilter =(MailFilterByFolderID)filter;
			query.append(" and folderID =" + mailFilter.getFolderID());
			//mailFilter.removeFilter();
			
		
		return query.toString();
	}

	private Mail buildMail(ResultSet results) throws Exception {
		final String id = String.valueOf(results.getInt(MAIL_COLUMN_ID));
		Mail mail = findCachedOrCreateNewMail(id);
		
		mail.setAttachments(results.getInt(MAIL_COLUMN_ATTACHMENT ));
		mail.setCcers(results.getString(MAIL_COLUMN_CC_NAME  ));
		mail.setBccers(results.getString(MAIL_COLUMN_BCC_NAME  ));
		mail.setDate(results.getString(MAIL_COLUMN_SEND_DATE  ));
		mail.setRead(results.getInt(MAIL_COLUMN_HAVE_READ ));
		mail.setSender(results.getString(MAIL_COLUMN_SENDER_NAME ));
		mail.setSubject(results.getString(MAIL_COLUMN_SUBJECT ));
		mail.setPriority(Mail.Priority.values()[results.getInt(MAIL_COLUMN_PRIORITY )]);
		mail.setMailflag(Mail.MailFlag.values()[results.getInt(MAIL_COLUMN_FLAG )]);
		mail.setMAILFORMAT(Mail.MailFormat.values()[results.getInt(MAIL_COLUMN_FORMAT )]);
		mail.setMailprivatetag(Mail.MailPrivateTag.values()[results.getInt(MAIL_COLUMN_TAG )]);
		mail.setFolderID(results.getInt(MAIL_COLUMN_FOLDERID ));
		mail.setId(results.getInt(MAIL_COLUMN_ID ));
		mail.setReceivers(results.getString(MAIL_COLUMN_RECEIVER_NAME ));
		mail.setAccountID(results.getInt(MAIL_COLUMN_ACCOUT_ID ));
		mail.setDataFilePath(results.getString(MAIL_COLUMN_MAILFILEPATH ));
		mail.setSize((results.getInt(MAIL_COLUMN_MESSAGE_SIZE)));
        mail.setUid(results.getString(MAIL_COLUMN_MESSAGE_UID));
				
		return mail;
	}
	
	/**
	 * Mails are cached by the backend in order to preserve identity.
	 * The cache uses weak references so that, should the cache
	 * be the only reference to the task, that task will be subject
	 * to garbage collection. In that event, the cache will be left
	 * with an empty slot (the key and the weak reference objects
	 * remain).
	 */
	private Mail findCachedOrCreateNewMail(String id) {
		Mail mail = (Mail)findInCache(Mail.class, id);
		if (mail == null) {
			mail = new Mail();
			mail.setId(Integer.parseInt(id));
			cache(Mail.class, id, mail);
		}
		return mail;
	}


	private void add(List<?> objects, Connection connection)
			throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = connection.prepareStatement(INSERT_MAIL_STATEMENT,
					Statement.RETURN_GENERATED_KEYS);
			for (Object object : objects) {

				Mail mail = (Mail) object;

				statement.setInt(MAIL_COLUMN_PRIORITY, mail.getPriority()
						.ordinal());
				statement.setInt(MAIL_COLUMN_FLAG, mail.getMailflag()
						.ordinal());
				statement.setInt(MAIL_COLUMN_FORMAT, mail.getMAILFORMAT()
						.ordinal());
				statement.setInt(MAIL_COLUMN_TAG, mail.getMailprivatetag()
						.ordinal());
				setString(statement, MAIL_COLUMN_SENDER_NAME, mail.getSender());
				setString(statement, MAIL_COLUMN_RECEIVER_NAME, mail
						.getReceivers());
				setString(statement, MAIL_COLUMN_CC_NAME, mail.getCcers());
				setString(statement, MAIL_COLUMN_BCC_NAME, mail.getBccers());
				setString(statement, MAIL_COLUMN_SUBJECT, mail.getSubject());
				statement.setInt(MAIL_COLUMN_ACCOUT_ID, mail.getAccountID());
				setString(statement, MAIL_COLUMN_MAILFILEPATH, mail
						.getDataFilePath());
				statement
						.setInt(MAIL_COLUMN_ATTACHMENT, mail.getAttachments());
				statement.setInt(MAIL_COLUMN_HAVE_READ, mail.getRead());
				statement.setInt(MAIL_COLUMN_FOLDERID, mail.getFolderID());
				statement.setInt(MAIL_COLUMN_MESSAGE_SIZE, mail.getSize());
				setString(statement, MAIL_COLUMN_SEND_DATE, mail.getDate());
				setString(statement, MAIL_COLUMN_MESSAGE_UID, mail.getUid());
				statement.execute();

				// Get the key for the added item.
				ResultSet keys = statement.getGeneratedKeys();
				keys.next();
				mail.setId(keys.getInt(1));
				String id = String.valueOf(mail.getId());
				cache(Mail.class, id, mail);
			}
		} finally {
			close(statement);
		}
	}

	@Override
	protected void update(Object object) {
		if (object instanceof Mail) updateMail((Mail)object);
	}
	
	private void updateMail(Mail mail) {
		Connection connection = null;
		PreparedStatement statement1 = null;
		try {
			connection = getConnection();
			statement1 = connection.prepareStatement(UPDATE_MAIL_STATEMENT);
			
			statement1.setInt(MAIL_COLUMN_PRIORITY, mail.getPriority().ordinal());
			statement1.setInt(MAIL_COLUMN_FLAG, mail.getMailflag().ordinal());
			statement1.setInt(MAIL_COLUMN_FORMAT, mail.getMAILFORMAT().ordinal());
			statement1.setInt(MAIL_COLUMN_TAG, mail.getMailprivatetag().ordinal());
			setString(statement1,MAIL_COLUMN_SENDER_NAME ,mail.getSender());
			setString(statement1,MAIL_COLUMN_RECEIVER_NAME,mail.getReceivers());
			setString(statement1,MAIL_COLUMN_CC_NAME ,mail.getCcers());
			setString(statement1,MAIL_COLUMN_BCC_NAME ,mail.getBccers());
			setString(statement1,MAIL_COLUMN_SUBJECT ,mail.getSubject());
			statement1.setInt(MAIL_COLUMN_ACCOUT_ID, mail.getAccountID());
			setString(statement1,MAIL_COLUMN_MAILFILEPATH ,mail.getDataFilePath());
			statement1.setInt(MAIL_COLUMN_ATTACHMENT, mail.getAttachments());											
			statement1.setInt(MAIL_COLUMN_HAVE_READ , mail.getRead());	
			statement1.setInt(MAIL_COLUMN_FOLDERID , mail.getFolderID());	
 			statement1.setInt(MAIL_COLUMN_MESSAGE_SIZE  , mail.getSize());
 			setString(statement1,MAIL_COLUMN_SEND_DATE  ,mail.getDate());
			statement1.setInt(MAIL_COLUMN_ID,mail.getId());
			setString(statement1,MAIL_COLUMN_MESSAGE_UID ,mail.getUid());
			statement1.execute();
		} catch (Exception e) {
			Activator.logError(e);
		} finally {
			close(statement1);
			close(connection);
		}
		
	}

	public int count(Filter filter) throws BackendException {
		if(!(filter instanceof MailFilter))
			return 0;
		Connection connection = null;
		try {
			connection = getConnection();
			return count(filter, connection);
		} catch (SQLException e) {
			throw new BackendException(e);

		} finally {
			close(connection);
		}
	}
	
	private int count(Filter filter, Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = generateStatement(filter, connection, COUNT_MAILS_STATEMENT);
			ResultSet results = statement.executeQuery();
			if (!results.next()) return 0;
			return results.getInt(1);
		} finally {
			close(statement);
		}
	}

	public void delete(List<?> objects) throws BackendException {
		PreparedStatement statement = null;
		Connection connection = null;
		try {
			connection = getConnection();
			statement = connection.prepareStatement(DELETE_MAIL_STATEMENT);
			for (Object object : objects) {
				Mail mail = (Mail) object;
				statement.setInt(1, mail.getId());
				statement.execute();
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			close(statement);
			close(connection);
		}
	}

	@Override
	protected void hookPropertyChangeListener(Object object, PropertyChangeListener propertyChangeListener) {
		if (object instanceof Mail) {
			((Mail)object).addPropertyChangeListener(propertyChangeListener);
		}		
	}


}
