/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.derby;

import java.beans.PropertyChangeListener;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import org.eclipse.samples.organizer.Filter;
import org.eclipse.samples.organizer.Visitor;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.models.Task;
import org.eclipse.samples.organizer.tasks.filters.TaskFilter;
 
// Changed by Ha Li, to separate different backend, such as task,mail,contact backend.
public class TaskDerbyBackend extends AbstractDatabaseBackend implements Backend {
	private static final String TASKS_TABLE_EXISTS_STATEMENT = "select * from organizer.tasks";
	private static final String CREATE_TASKS_TABLE_STATEMENT = "create table organizer.tasks (id int generated always as identity primary key, due_date date, reminder_date date, complete char(1), priority int, subject varchar(512), details clob)";
	
	private static final String SELECT_TASKS_STATEMENT = "select due_date, reminder_date, complete, priority, subject, details, id from organizer.tasks where 1=1";
	private static final String INSERT_TASK_STATEMENT = "insert into organizer.tasks (due_date, reminder_date, complete, priority, subject, details, id) values (?, ?, ?, ?, ?, ?, DEFAULT)";
	private static final String UPDATE_TASK_STATEMENT = "update organizer.tasks set due_date=?, reminder_date=?, complete=?, priority=?, subject=?, details=? where id=?";
	private static final String COUNT_TASKS_STATEMENT = "select count(*) from organizer.tasks where 1=1";
	
	private static final int COLUMN_DUE_DATE = 1;
	private static final int COLUMN_REMINDER_DATE = 2;
	private static final int COLUMN_COMPLETE = 3;
	private static final int COLUMN_PRIORITY = 4;
	private static final int COLUMN_SUBJECT = 5;
	private static final int COLUMN_DETAILS = 6;
	private static final int COLUMN_ID = 7;
	
	public TaskDerbyBackend(Database database) {
		super(database);
	}

	public String getName() {
		return "Local Task Organizer Database";
	}
		
	@Override
	protected boolean isDatabaseConfigured(Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = connection.prepareStatement(TASKS_TABLE_EXISTS_STATEMENT);
			statement.setMaxRows(1);
			statement.execute();
			return true;
		} catch (Exception e) {
			// TODO If it is not a "table not found" exception, rethrow.
			return false;
		} finally {
			close(statement);
		}
	}
	
	@Override
	protected void createTables(Connection connection) throws SQLException {
		Statement statement = null;
		try {
			statement = connection.createStatement();
			statement.execute(CREATE_TASKS_TABLE_STATEMENT);
		} finally {
			close(statement);
		}
	}
	
	public void add(List<?> objects) throws BackendException {
		Connection connection = null;
		try {
			connection = getConnection();
			add(objects, connection);
		} catch (SQLException e) {
			throw new BackendException(e);
		} finally {
			close(connection);
		}
	}

	public void find(Filter filter, Visitor visitor) {
		if(!(filter instanceof TaskFilter))
			return;
		Connection connection = null;
		try {
			connection = getConnection();
			find(filter, visitor, connection);	
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			close(connection);
		}
	}

	private void find(Filter filter, Visitor visitor, Connection connection) throws Exception {
		cleanCache();
		if (filter.includesType(Task.class)) {
			findTasks(filter,visitor, connection);
		}
	}
	
	private void findTasks(Filter filter, Visitor visitor, Connection connection) throws Exception {
		PreparedStatement statement = null;
		try {
			statement = generateStatement(filter, connection, SELECT_TASKS_STATEMENT);
			ResultSet results = statement.executeQuery();
			while (results.next()) {
				Task task = buildTask(results);
				visitor.visit(task);
			}
		} finally {
			close(statement);
		}
	}

	// TODO Need to actually consider filter.
	private PreparedStatement generateStatement(Filter filter, Connection connection, String baseQuery) throws SQLException {
		String query = buildQuery(filter, baseQuery);
		PreparedStatement statement = connection.prepareStatement(query);
		return statement;
	}
	
	private String buildQuery(Filter filter, String baseQuery) {
		StringBuilder query = new StringBuilder(baseQuery);
		if (filter instanceof TaskFilter) {
			TaskFilter taskFilter = (TaskFilter)filter;
			if (taskFilter.isHideCompleteTasks()) {
				query.append(" and complete='n'");
			}
		}
		return query.toString();
	}

	private Task buildTask(ResultSet results) throws Exception {
		final String id = String.valueOf(results.getInt(COLUMN_ID));
		Task task = findCachedOrCreateNewTask(id);
		
		task.setDueDate(getDate(results, COLUMN_DUE_DATE));
		task.setReminderDate(getDate(results, COLUMN_REMINDER_DATE));
		task.setComplete("y".equals(results.getString(COLUMN_COMPLETE)));
		task.setPriority(Task.Priority.values()[results.getInt(COLUMN_PRIORITY)]);
		task.setSubject(results.getString(COLUMN_SUBJECT));
		task.setDetails(getClob(results, COLUMN_DETAILS));
				
		return task;
	}
	
	/**
	 * Tasks are cached by the backend in order to preserve identity.
	 * The cache uses weak references so that, should the cache
	 * be the only reference to the task, that task will be subject
	 * to garbage collection. In that event, the cache will be left
	 * with an empty slot (the key and the weak reference objects
	 * remain).
	 */
	private Task findCachedOrCreateNewTask(String id) {
		Task task = (Task)findInCache(Task.class, id);
		if (task == null) {
			task = new Task();
			task.setId(id);
			cache(Task.class, id, task);
		}
		return task;
	}


	private void add(List<?> objects, Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = connection.prepareStatement(INSERT_TASK_STATEMENT, Statement.RETURN_GENERATED_KEYS);
			for (Object object : objects) {
				if (object instanceof Task) {
					Task task = (Task)object; 
					setDate(statement, COLUMN_DUE_DATE, task.getDueDate());
					setDate(statement, COLUMN_REMINDER_DATE, task.getReminderDate());
					statement.setString(COLUMN_COMPLETE, task.isComplete() ? "y" : "n");
					statement.setInt(COLUMN_PRIORITY, task.getPriority().ordinal());
					setString(statement, COLUMN_SUBJECT, task.getSubject());
					setClob(statement, COLUMN_DETAILS, task.getDetails());
					statement.execute();
				
					// Get the key for the added item.
					ResultSet keys = statement.getGeneratedKeys();
					keys.next();
					String id = String.valueOf(keys.getInt(1));
					task.setId(id);
					cache(Task.class, id, task);
				} else {
					// TODO Throw an UnsupportedObjectException or something.
				}
			}
		} finally {
			close(statement);
		}
	}

	@Override
	protected void update(Object object) {
		if (object instanceof Task) updateTask((Task)object);
	}
	
	private void updateTask(Task task) {
		Connection connection = null;
		PreparedStatement statement = null;
		try {
			connection = getConnection();
			statement = connection.prepareStatement(UPDATE_TASK_STATEMENT);
			setDate(statement, COLUMN_DUE_DATE, task.getDueDate());
			setDate(statement, COLUMN_REMINDER_DATE, task.getReminderDate());
			statement.setString(COLUMN_COMPLETE, task.isComplete() ? "y" : "n");
			statement.setInt(COLUMN_PRIORITY, task.getPriority().ordinal());
			statement.setString(COLUMN_SUBJECT, task.getSubject());
			setClob(statement, COLUMN_DETAILS, task.getDetails());
			statement.setInt(COLUMN_ID, Integer.parseInt(task.getId()));
			statement.execute();
		} catch (Exception e) {
			Activator.logError(e);
		} finally {
			close(statement);
			close(connection);
		}
	}

	public int count(Filter filter) throws BackendException {
		if(!(filter instanceof TaskFilter))
			return 0;
		Connection connection = null;
		try {
			connection = getConnection();
			return count(filter, connection);
		} catch (SQLException e) {
			throw new BackendException(e);

		} finally {
			close(connection);
		}
	}
	
	private int count(Filter filter, Connection connection) throws SQLException {
		PreparedStatement statement = null;
		try {
			statement = generateStatement(filter, connection, COUNT_TASKS_STATEMENT);
			ResultSet results = statement.executeQuery();
			if (!results.next()) return 0;
			return results.getInt(1);
		} finally {
			close(statement);
		}
	}

	public void delete(List<?> objects) throws BackendException {
		// TODO Auto-generated method stub
		
	}

	@Override
	protected void hookPropertyChangeListener(Object object, PropertyChangeListener propertyChangeListener) {
		if (object instanceof Task) {
			((Task)object).addPropertyChangeListener(propertyChangeListener);
		}		
	}

}
