/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * This class is derived from code contributed by Bahadir Yagan in
 * bug 19945 (https://bugs.eclipse.org/bugs/show_bug.cgi?id=19945#c11), 
 * though this implementation is almost a complete rewrite (little of the 
 * original code remains).
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.calendar;

import java.util.Date;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

import com.ibm.icu.text.DateFormatSymbols;
import com.ibm.icu.util.Calendar;
import com.ibm.icu.util.ULocale;

class DatePicker extends Composite {
	private Date date = null;
	
	private DateFormatSymbols dateFormatSymbols = null;
	
	private Label[] cells = null;
	
	private int year;	
	private int month;
	
	/**
	 * This is the cell containing the selected date.
	 */
	private Label selectedCell;
	
	/** 
	 * This is the highlighted cell. That is, this cell is not necessary
	 * the selected one (i.e. the cell containing the selected date), but
	 * is the cell under the cursor.
	 */
	private Label highlightedCell;
	
	/**
	 * This is an instance of Calendar for arbitrary use.
	 */
	private Calendar calendar;

	private Label headerLabel;

	private Composite body;

	private ULocale locale;

	public DatePicker(Composite parent, int style, ULocale locale) {
		super(parent, style);
		this.locale = locale;
		initialize();
		updateCalendar();
	}

	public DatePicker(Composite parent, int style) {
		this(parent, style, ULocale.getDefault());
	}

	private void initialize() {
		// Initialize the calendar and use the year and month
		// it represents as arbitrary starting points for the
		// month to initially display. These values will
		// be overridden when the date is set.
				
		calendar = Calendar.getInstance(locale);
		year = calendar.get(Calendar.YEAR);
		month = calendar.get(Calendar.MONTH);
		
		dateFormatSymbols = new DateFormatSymbols(locale);
		cells = new Label[42];
		
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;
		this.setLayout(gridLayout);

		Composite navigation = createNavigation(this);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		navigation.setLayoutData(gridData);
		
		Composite header = createHeader(this);
		gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		header.setLayoutData(gridData);
		
		body = createBody(this);
		gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		body.setLayoutData(gridData);
		
		addTraverseListener(new TraverseListener() {
			public void keyTraversed(TraverseEvent e) {
				e.doit = true;
			}			
		});
		
		pack();
	}

	/**
	 * This method creates the navigation area at the top of the
	 * panel. It is composed of several arrow buttons (for
	 * changing the month and year) and a label that indicates
	 * the current month and year.
	 */
	private Composite createNavigation(Composite parent) {
		Composite navigation = new Composite(parent, SWT.NONE);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		gridLayout.makeColumnsEqualWidth = false;
		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;
		navigation.setLayout(gridLayout);	
		
		Button previousYear = new Button(navigation, SWT.ARROW | SWT.LEFT);
		previousYear.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				changeMonth(-(calendar.getMaximum(Calendar.MONTH) + 1)); // argument used to be -12
			}			
		});
		
		Button previousMonth = new Button(navigation, SWT.ARROW | SWT.LEFT);
		previousMonth.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				changeMonth(-(calendar.getMinimum(Calendar.MONTH) + 1));	// argument used to be -1
			}			
		});
		
		headerLabel = new Label(navigation, SWT.NONE);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.minimumWidth = 100;
		headerLabel.setLayoutData(gridData);
		
		headerLabel.setAlignment(SWT.CENTER);
		headerLabel.setText(""); //$NON-NLS-1$
		
		Button nextMonth = new Button(navigation, SWT.ARROW | SWT.RIGHT);
		nextMonth.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				changeMonth(calendar.getMinimum(Calendar.MONTH) + 1);	// argument used to be 1
			}			
		});
		
		Button nextYear = new Button(navigation, SWT.ARROW | SWT.RIGHT);
		nextYear.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				changeMonth(calendar.getMaximum(Calendar.MONTH) + 1);	// argument used to be 12
			}			
		});
		return navigation;
	}

	/**
	 * This method initializes composite
	 * 
	 */
	private Composite createHeader(Composite parent) {
		Composite header = new Composite(parent, SWT.NONE);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 7;
		gridLayout.makeColumnsEqualWidth = true;
		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		header.setLayout(gridLayout);	
		
		// The array that's returned by getShortWeekdays() has no value in the first
		// position (0); for sanity later, exclude that value.
		// TODO This should probably be cached.
		String[] weekDays = new String[7];
		System.arraycopy(dateFormatSymbols.getShortWeekdays(), 1, weekDays, 0, 7);
		GridData labelGridData = new GridData();
		labelGridData.horizontalAlignment = SWT.FILL;
		labelGridData.grabExcessHorizontalSpace = true;
		labelGridData.verticalAlignment = SWT.FILL;
		// FIXME Sort out why the labels aren't reporting proper sizes back.
		labelGridData.minimumWidth = 25;
		for (int i = 0; i < 7; ++i) {
			Label headerLabel = new Label(header, SWT.CENTER);
			//headerLabel.setText(weekDays[i].substring(0,1));
			headerLabel.setText(weekDays[(i + calendar.getFirstDayOfWeek() - Calendar.SUNDAY) % 7]);
			headerLabel.setAlignment(SWT.CENTER);
			headerLabel.setLayoutData(labelGridData);
		}
		
		return header;
	}

	/**
	 * This method initializes composite1
	 * 
	 */
	private Composite createBody(Composite parent) {
		final Composite body = new Composite(parent, SWT.BORDER);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 7;
		gridLayout.marginLeft = 0;
		gridLayout.makeColumnsEqualWidth = true;
		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		body.setLayout(gridLayout);
		
		body.setBackground(getColorWhite());
		
		createCalendarData(body);
		body.addListener (SWT.Traverse, new Listener () {
			public void handleEvent (Event event) {
				notifyListeners(SWT.Traverse, event);
			}
		});
//		body.addPaintListener(new PaintListener() {
//			public void paintControl(PaintEvent e) {
//				if (!body.isFocusControl()) return;
//				Rectangle bounds = body.getClientArea();
//				e.gc.setForeground(getColorSelected());
//				e.gc.setLineDash(new int[] {1, 1});
//				e.gc.drawRectangle(0, 0, bounds.width-1, bounds.height-1);
//			}
//			
//		});
		body.addFocusListener(new FocusListener() {

			public void focusGained(FocusEvent e) {
				body.redraw();
			}

			public void focusLost(FocusEvent e) {
				body.redraw();
			}
			
		});
		body.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				switch (e.keyCode) {
				case SWT.ARROW_LEFT : 
					changeHighlightedCell(-1);
					break;
				case SWT.ARROW_RIGHT :
					changeHighlightedCell(1);
					break;
				case SWT.ARROW_UP :
					changeHighlightedCell(-7);
					break;
				case SWT.ARROW_DOWN :
					changeHighlightedCell(7);
					break;
				case SWT.CR :
					if (highlightedCell != null) {
						setDate((Date)highlightedCell.getData());
					}
					break;
				}
			}
		});
		
		return body;
	}

	@Override
	public boolean setFocus() {
		return body.setFocus();
	}
	
	private void createCalendarData(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = SWT.FILL;
		gridData.grabExcessVerticalSpace = true;
		// FIXME Get rid of the magic number
		for (int index = 0; index < 42; ++index) {
			Composite cell = new Composite(parent, SWT.BORDER);
			cell.setLayoutData(gridData);
			cell.setBackground(getColorWhite());
			cell.setLayout(new GridLayout(1, true));
			Label label = new Label(cell, SWT.RIGHT);
			label.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
			label.setText("999"); //$NON-NLS-1$
			label.setData("index", index); //$NON-NLS-1$
			cells[index] = label;
			cell.addMouseListener(new MouseAdapter() {
				public void mouseDown(MouseEvent arg0) {
					Label label = (Label) arg0.getSource();
					setHighlightCell(label);
				}

//				public void mouseUp(MouseEvent arg0) {
//					Label label = (Label) arg0.getSource();
//					setDate((Date)label.getData());
//				}
			});
		}
	}
	
	private boolean isHighlightCell(Label label) {
		return highlightedCell == label;
	}

	private boolean isSelectedCell(Label cell) {
		return selectedCell == cell;
	}
	
	private boolean isCurrentMonthCell(Label label) {
		calendar.setTimeInMillis(((Date)label.getData()).getTime());
		return calendar.get(Calendar.MONTH) == month;
	}

	private boolean areSameDate(Date date0, Date date1) {
		if (date0 == null) return false;
		if (date1 == null) return false;
		
		Calendar calendar0 = Calendar.getInstance(locale);
		calendar0.setTimeInMillis(date0.getTime());
		Calendar calendar1 = Calendar.getInstance(locale);
		calendar1.setTimeInMillis(date1.getTime());

		if (calendar0.get(Calendar.YEAR) != calendar1.get(Calendar.YEAR)) return false;
		if (calendar0.get(Calendar.MONTH) != calendar1.get(Calendar.MONTH)) return false;
		if (calendar0.get(Calendar.DATE) != calendar1.get(Calendar.DATE)) return false;
		
		return true;
	}
	
	private void updateCalendar() {		
		headerLabel.setText(getHeaderLabelText());
		
		selectedCell = null;

		Calendar calendar = Calendar.getInstance(locale);
		calendar.set(year, month, 1);
		int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK) - calendar.getFirstDayOfWeek();
		calendar.add(Calendar.DAY_OF_MONTH, -dayOfWeek);
		
		for(int index=0;index<cells.length;index++) {
			Date time = calendar.getTime();
			Label label = cells[index];
			label.setData(time);
			int dayOfMonth = calendar.get(Calendar.DAY_OF_MONTH);
			String text = String.valueOf(dayOfMonth);
			
			if (dayOfMonth == 1 | index==0) {
				int monthIndex = calendar.get(Calendar.MONTH);
				String month = dateFormatSymbols.getMonths(DateFormatSymbols.STANDALONE, DateFormatSymbols.WIDE)[monthIndex];
				text = month + " " + text; 
			}
			
			label.setText(text);			
			if (areSameDate(time, date)) selectedCell = label;
			
			decorateCell(label);
			
			calendar.add(Calendar.DAY_OF_MONTH, 1);
		}
	}

	private String getHeaderLabelText() {
		StringBuilder builder = new StringBuilder();
		// The STANDALONE allows for month names not to be conjugated
		builder.append(dateFormatSymbols.getMonths(DateFormatSymbols.STANDALONE, DateFormatSymbols.WIDE)[month]);
		builder.append(" "); //$NON-NLS-1$
		builder.append(year);
		return builder.toString();
	}

	private void decorateCell(Label label) {
		if (label == null) return;
		if (isSelectedCell(label)) {
			decorateCurrentDateCell(label);
		} else if (isHighlightCell(label)) {
			decorateHighlightCell(label);
		} else {
			undecorateCell(label);
		}
	}

	private void undecorateCell(Label label) {
		label.setForeground(isCurrentMonthCell(label) ? getColorBlack() : getColorGray());
		label.setBackground(getColorWhite());
	}

	private void decorateCurrentDateCell(Label label) {		
		label.setForeground(getColorSelectedText());
		label.setBackground(getColorSelected());
	}
	
	private void decorateHighlightCell(Label label) {
		label.setForeground(getColorWhite());
		label.setBackground(getColorGray());
	}

	public void setDate(Date date) {
		//Date oldDate = this.date;
		this.date = date;
		if (date != null) {
			calendar.setTimeInMillis(date.getTime());
			month = calendar.get(Calendar.MONTH);
			year = calendar.get(Calendar.YEAR);
		}
		updateCalendar();
		// TODO When a date is selected, we need to set the workbench selection
	}
	
	public Date getDate() {
		return date;
	}

	protected void setHighlightCell(Label label) {
		Label oldHighlightCell = highlightedCell;
		highlightedCell = label;
		
		decorateCell(oldHighlightCell);
		decorateCell(highlightedCell);
	}
	
	protected void changeMonth(int delta) {
		Calendar calendar = Calendar.getInstance(locale);
		calendar.set(year, month, 1);
		calendar.add(Calendar.MONTH, delta);
		year = calendar.get(Calendar.YEAR);
		month = calendar.get(Calendar.MONTH);
		updateCalendar();
	}
	
	protected void changeHighlightedCell(int delta) {
		int index = 0;
		if (highlightedCell == null) highlightedCell = selectedCell;
		if (highlightedCell != null) {
			index = (Integer)highlightedCell.getData("index") + delta; //$NON-NLS-1$
			if (index < 0) {
				changeMonth(-1);
				index = 42 + index;
			}
			if (index >= 42) {
				changeMonth(1);
				index = index - 42;
			}
		}
		setHighlightCell(cells[index]);
	}

	/**
	 * @category colours
	 */
	private Color getColorBlack() {
		return this.getDisplay().getSystemColor(SWT.COLOR_BLACK);
	}

	/**
	 * @category colours
	 */
	private Color getColorWhite() {
		return this.getDisplay().getSystemColor(SWT.COLOR_WHITE);
	}

	/**
	 * @category colours
	 */
	private Color getColorGray() {
		return this.getDisplay().getSystemColor(SWT.COLOR_GRAY);
	}

	/**
	 * @category colours
	 */
	private Color getColorSelectedText() {
		return this.getDisplay().getSystemColor(SWT.COLOR_LIST_SELECTION_TEXT);
	}

	/**
	 * @category colours
	 */
	private Color getColorSelected() {
		return this.getDisplay().getSystemColor(SWT.COLOR_LIST_SELECTION);
	}

	public ULocale getLocale() {
		return locale;
	}

	public void setLocale(ULocale locale) {
		this.locale = locale;
	}
}