/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*******************************************************************************/
package org.eclipse.samples.organizer.ui.contact.dnd;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.samples.organizer.models.Contact;
import org.eclipse.samples.organizer.ui.contact.Activator;
import org.eclipse.samples.organizer.ui.contact.wizards.ImportContactsWizard;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Control;


public class ContactDropTarget {

	private DropHandler[] dropHandlers = findDropHandlers();
	
	public ContactDropTarget(final Control control) {
		DropTarget dropTarget = new DropTarget(control, DND.DROP_COPY | DND.DROP_DEFAULT);
		dropTarget.setTransfer(getSupportedTransfers());
		dropTarget.addDropListener(new DropTargetAdapter() {
			public void dragEnter(DropTargetEvent event) {
				if (event.detail == DND.DROP_DEFAULT)
					event.detail = DND.DROP_COPY;
			}

			public void dragOperationChanged(DropTargetEvent event) {
				if (event.detail == DND.DROP_DEFAULT)
					event.detail = DND.DROP_COPY;
			}

			public void dragOver(DropTargetEvent event) {
			}

			public void drop(DropTargetEvent event) {
				for (DropHandler handler : dropHandlers) {
					ArrayList<Contact> contacts = (ArrayList<Contact>) handler.createContactsFromDropEvent(event);
					if (contacts != null) {
						ImportContactsWizard.importContacts(control.getShell(), contacts);
						break;
					}
				}
			}
		});
	}

	private Transfer[] getSupportedTransfers() {
		// Use a list instead of a set because order is important!
		ArrayList<Transfer> transfers = new ArrayList<Transfer>();
		for (DropHandler handler : dropHandlers) {
			for(Transfer transfer : handler.getSupportedTransfers()) {
				if (!transfers.contains(transfer)) transfers.add(transfer);
			}
		}
		return transfers.toArray(new Transfer[transfers.size()]);
	}
	
	private DropHandler[] findDropHandlers() {
		List<DropHandler> handlers = new ArrayList<DropHandler>();
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(Activator.getSymbolicName() + ".drop_handler");
		for (IConfigurationElement config : point.getConfigurationElements()) {
			if ("drop_handler".equals(config.getName())) {
				try {
					Object object = config.createExecutableExtension("class");
					if (object instanceof DropHandler) {
						handlers.add((DropHandler) object);
					}
				} catch (CoreException e) {
					Activator.logError(e.getStatus());
				}					
			}
		}
		// Sort the handlers by their weight. Higher weights first.
		Collections.sort(handlers, new Comparator<DropHandler>() {
			public int compare(DropHandler handler1, DropHandler handler2) {
				return handler2.getWeight().compareTo(handler1.getWeight());
			}
			
		});
		return handlers.toArray(new DropHandler[handlers.size()]);
	}
}
