/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*******************************************************************************/
package org.eclipse.samples.organizer.ui.contact.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.Visitor;
import org.eclipse.samples.organizer.events.ContentChangeEvent;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.listeners.ContentChangeListener;
import org.eclipse.samples.organizer.models.Contact;
import org.eclipse.samples.organizer.contacts.filters.ContactFilter;
import org.eclipse.samples.organizer.ui.contact.Activator;
import org.eclipse.samples.organizer.ui.contact.dnd.ContactDropTarget;
import org.eclipse.samples.organizer.ui.contact.wizards.ImportContactsWizard;
import org.eclipse.samples.organizer.ui.mail.actions.CreateMailAction;
import org.eclipse.samples.organizer.ui.util.CellDrawEvent;
import org.eclipse.samples.organizer.ui.util.CellDrawListener;
import org.eclipse.samples.organizer.ui.util.CellMeasureEvent;
import org.eclipse.samples.organizer.ui.util.CellMeasureListener;
import org.eclipse.samples.organizer.ui.util.SuperTableViewer;
import org.eclipse.samples.organizer.ui.util.TableViewerColumnDefinition;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.progress.WorkbenchJob;

public class ContactView extends ViewPart {
	private SuperTableViewer viewer;

	private Action openAction;
	private Action deleteAction;
	private Action createAction;
	private Action refreshAction;
	
	private TableViewerColumnDefinition phoneColumn;
	private TableViewerColumnDefinition nameColumn;
	private TableViewerColumnDefinition emailColumn;
			
	private IMemento memento;

	private SashForm sashForm;

	private Composite buttonsComposite;

	class ViewContentProvider implements IStructuredContentProvider {
		private List<Contact> contacts = new ArrayList<Contact>();
		
		private ContentChangeListener contentChangeListener = new ContentChangeListener() {
			public void contentChanged(ContentChangeEvent event) {
				viewer.refresh();
			}
		};
		
		private PropertyChangeListener propertyChangeListener = new PropertyChangeListener() {
			/*
			 * We go a little overboard here. When a property on one of the 
			 * contacts changes, update the entire table. The change may affect
			 * the sort order.
			 * TODO Is there a way to update just the affected row and force a resort?
			 */
			public void propertyChange(PropertyChangeEvent event) {
				viewer.refresh();
			}
		};
		
		public ViewContentProvider() {
			hookContentListeners();
		}

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}

		public void dispose() {
			unhookContentListeners();
			unhookChangeListener();
		}
		
		/**
		 * Hooks a content change listener to the organizer manager so that
		 * whenever content is changed, the receiver gets a chance to update.
		 * Likewise, add property change listener to the ContactFilter so that
		 * when it changes, the receiver can update.
		 */
		private void hookContentListeners() {			
			OrganizerManager.instance.addContentChangeListener(contentChangeListener);
			ContactFilter.instance.addPropertyChangeListener(propertyChangeListener);
		}

		private void unhookContentListeners() {
			OrganizerManager.instance.removeContentChangeListener(contentChangeListener);
			ContactFilter.instance.removePropertyChangeListener(propertyChangeListener);
		}		
		
		public Object[] getElements(Object parent) {
			unhookChangeListener();
			
			/*
			 * Spawn  a job to find the contacts. 
			 */
			Job job = new Job("Find Contacts Job") {
				@Override
				protected IStatus run(IProgressMonitor monitor) {
					try {
						fetchContacts();
					} catch (BackendException e) {
						Activator.logError(e);
					}
					return Status.OK_STATUS;
				}
				
			};
			job.schedule();
			
			return new Object[0];
		}
		
		/**
		 * Hook a property change listener to the given contact. Should the
		 * contact change, the view will be updated.
		 */
		private void hookPropertyChangeListener(Contact contact) {
			contact.addPropertyChangeListener(propertyChangeListener);
		}
		
		private void addContactToView(final Contact contact) {
			/*
			 * Spawn a job to do the update; this will ensure that
			 * the workbench is open, and that the update operation
			 * occurs in the UI thread.
			 */
			WorkbenchJob job = new WorkbenchJob("Updating the view") {
				@Override
				public IStatus runInUIThread(IProgressMonitor monitor) {
					viewer.add(contact);
					return Status.OK_STATUS;
				}				
			};
			job.schedule();
		}
		
		/**
		 * Stop listening to property change events on all the contacts
		 * we've encountered so far.
		 */
		private void unhookChangeListener() {
			if (contacts == null) return;
			for (Contact contact : contacts) {
				contact.removePropertyChangeListener(propertyChangeListener);
			}
		}

		private void fetchContacts() throws BackendException {
			OrganizerManager.instance.find(ContactFilter.instance, new Visitor() {
				public void visit(Object object) {
					Contact contact = (Contact)object;
					hookPropertyChangeListener(contact);
					contacts.add(contact);
					addContactToView(contact);
				}						
			});
		}
	}

	public void createPartControl(Composite parent) {
		sashForm = new SashForm(parent, SWT.HORIZONTAL);
		buttonsComposite = new Composite(sashForm, SWT.SMOOTH);
		createButtonsArea(buttonsComposite);
		viewer = new SuperTableViewer(sashForm, SWT.FULL_SELECTION | SWT.MULTI
				| SWT.H_SCROLL | SWT.V_SCROLL);
		sashForm.setWeights(new int[]{50,parent.getShell().getBounds().width -20});
		Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		new ContactDropTarget(viewer.getTable());
		
		viewer.setContentProvider(new ViewContentProvider());
		getSite().setSelectionProvider(viewer);
		
		//makeActions();
		
	//	hookContextMenu();
		hookDoubleClickAction();
	//	contributeToActionBars();
		
		// Restore the state of the table; this will create the columns.
		restoreState();
				
		hookListeners();
		viewer.setInput(getViewSite());
	
		//System.out.print();
	}
	
	private void createButtonsArea(final Composite buttonsarea) {
	
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing=0;
		gridLayout.verticalSpacing =0 ;
		gridLayout.numColumns = 1;
		gridLayout.horizontalSpacing = 1;
		buttonsarea.setLayout(gridLayout);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 1;
		gridData.grabExcessHorizontalSpace = true;
		
		Button button = new Button(buttonsarea,SWT.TOGGLE);
		button.setLayoutData(gridData);
		button.setText("All");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ContactFilter.instance.setNameStart('*');
				viewer.refresh();
			}

		});
		char[] buttonChar = new char[]{'A','B','C','D','E','F','G','H','I','G','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z'};
		for(int i=0, len = buttonChar.length; i<len; i++){
		final Button buttonTemp = new Button(buttonsarea,SWT.TOGGLE);
		buttonTemp.setLayoutData(gridData);
		buttonTemp.setText(String.valueOf(buttonChar[i]));
		buttonTemp.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				ContactFilter.instance.setNameStart(((Button) e.widget).getText().charAt(0));
				//viewer.refresh();
				for(int j =0, childlen = buttonsarea.getChildren().length;j <childlen; j++){
					if(buttonsarea.getChildren()[j] instanceof Button){
						((Button)buttonsarea.getChildren()[j]).setSelection(false);
						buttonTemp.setSelection(true);
					}
				}
			}

		});
		}

	}

	@Override
	public void saveState(IMemento memento) {
		// Save the column state.
		IMemento columnsMemento = memento.createChild("columns");
		for (TableViewerColumnDefinition column : viewer.getColumnDefinitions()) {
			IMemento columnMemento = columnsMemento.createChild("column");
			columnMemento.putString("id", column.getId());
			columnMemento.putInteger("width", column.getWidth());
			columnMemento.putString("visible", String.valueOf(column.isVisible()));
		}
		// Save the sort state.
		IMemento sortMemento = memento.createChild("sort");
		sortMemento.putString("column", viewer.getSortColumnId());
		sortMemento.putString("direction", viewer.getSortDirection() == SWT.UP ? "UP" : "DOWN");
	}
	
	@Override
	public void init(IViewSite site, IMemento memento) throws PartInitException {
		this.memento = memento;
		super.init(site, memento);	
	}

	private void hookListeners() {
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				//updateActions();
			}
		});
	}
	
	private void updateActions() {
		IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
		boolean isAnythingSelected = selection.size() > 0;
		deleteAction.setEnabled(isAnythingSelected);
		openAction.setEnabled(isAnythingSelected);
//		viewer.update(selection, null);
	}

	private void importTasks(final ArrayList<Contact> contacts) {
		ImportContactsWizard.importContacts(getSite().getShell(), contacts);
	}
	
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void fillContextMenu(IMenuManager manager) {
		manager.add(openAction);
		manager.add(createAction);
		manager.add(deleteAction);
		manager.add(refreshAction);
		// Create and populate a submenu that the user can use
		// to show/hide columns.
		IMenuManager columnsMenu = new MenuManager("Columns");
		// TODO Fill the columns menu
		viewer.fillColumnsMenu(columnsMenu);
		manager.add(columnsMenu);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void makeActions() {
		makeOpenAction();
		makeDeleteAction();
		makeCreateAction();
		makeRefreshAction();
		
		updateActions();
	}

	private void makeOpenAction() {
		openAction = new Action() {	
			@Override
			public void run() {
				Contact contact = (Contact)((IStructuredSelection)viewer.getSelection()).getFirstElement();
				IWorkbenchPage page = getSite().getPage(); 
			/*	try {
				//	page.openEditor(new ContactEditorInput(contact), "org.eclipse.organizer.contact.ui.editors.TaskEditor");
				} catch (PartInitException e) {
					// TODO Message to user? Log this at least.
					e.printStackTrace();
				}*/
			}
		};
		openAction.setText("Open");
		openAction.setToolTipText("Open/edit this contact");
	}

	private void makeDeleteAction() {
		deleteAction = new Action() {
			@SuppressWarnings("unchecked")
			public void run() {
				ISelection selection = viewer.getSelection();
				List<Contact> contacts = (List<Contact>)((IStructuredSelection)selection).toList();
				if (contacts == null) return;
				if (contacts.size() == 0) return;
				OrganizerManager.instance.delete(contacts);
		        viewer.refresh(false);
			}	        
		};
		deleteAction.setText("Delete");
		deleteAction.setToolTipText("Delete the currently selected contact(s)");
		deleteAction.setImageDescriptor(getSharedImage(ISharedImages.IMG_TOOL_DELETE));
		deleteAction.setDisabledImageDescriptor(getSharedImage(ISharedImages.IMG_TOOL_DELETE_DISABLED));
		//deleteAction.setActionDefinitionId(ActionFactory.DELETE.getId());
	}

	private void makeCreateAction() {
		createAction = new Action() {
			public void run() {
				ArrayList<Contact> list = new ArrayList<Contact>();
				Contact newTask = new Contact();
				list.add(newTask);
				importTasks(list);
			}
		};
		createAction.setText("New contact...");
		createAction.setToolTipText("Creates a new contact.");
		createAction.setImageDescriptor(Activator.getDefault().getImageDescriptor(Activator.IMAGE_NEW_TASK));
		createAction.setDisabledImageDescriptor(getSharedImage(ISharedImages.IMG_TOOL_NEW_WIZARD_DISABLED));
	}
	
	private void makeRefreshAction() {
		refreshAction = new Action() {
			public void run() {
				viewer.refresh();
			}
		};
		refreshAction.setText("Refresh");
		refreshAction.setToolTipText("Refreshes the contact list.");
		refreshAction.setImageDescriptor(Activator.getDefault().getImageDescriptor(Activator.IMAGE_REFRESH));
	}
	
	private ImageDescriptor getSharedImage(String id) {
		return PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(id);
	}
	
	private void restoreState() {
		restoreColumnState();
		restoreSortState();
	}
	
	/**
	 * This method makes the columns. It consults with the memento
	 * (provided by a prior call of the init() method) and builds the columns identified
	 * therein. Finally, it builds all the columns that had not been represented in
	 * the memento. Once the columns are built, the sort column and direction are
	 * restored. We do all this to handle the case where some future version of
	 * this class adds (or even removes) columns. 
	 */
	private void restoreColumnState() {
		/*
		 * Build a list of the ids of all the columns that we want to build. As columns
		 * are built, their id is removed from this list so that we know--after the
		 * columns indicated in the memento are built--what other columns remain.
		 */
		List<String> idsOfColumnsThatNeedToBeBuilt = new ArrayList<String>();
		idsOfColumnsThatNeedToBeBuilt.addAll(Arrays.asList(new String[] { "contact", "name", "email", "mobilephone" }));
		if (memento != null) {
			IMemento columnsMemento = memento.getChild("columns");
			if (columnsMemento != null) {
				IMemento columnMementos[] = columnsMemento.getChildren("column");
				for (IMemento columnMemento : columnMementos) {
					/*
					 * For each column mentioned in the memento, get the
					 * stored values (id, width, visibility). Use this
					 * information to rebuild the column.
					 */
					String id = columnMemento.getString("id");
					if (id == null) continue;
					int width = Integer.valueOf(columnMemento.getInteger("width"));
					boolean visible = Boolean.valueOf(columnMemento.getString("visible"));
					makeColumn(id, width, visible);
					
					// Remove this column from the list of columns that need to be built.
					idsOfColumnsThatNeedToBeBuilt.remove(id);
				}
			}
		}
		
		// Build any columns that we did not find in the memento.
		for (String id : idsOfColumnsThatNeedToBeBuilt) {
			makeColumn(id, null, null);
		}
		
		restoreSortState();
	
		memento = null;
	}

	/**
	 * This method restores the sort state of the table based on the
	 * information stored in the memento.
	 */
	private void restoreSortState() {
		String sortColumn = "due";
		int sortDirection = SWT.UP;
		
		/*
		 * Get the sort information from the memento if it is available.
		 */
		if (memento != null) {
			IMemento sortMemento = memento.getChild("sort");
			if (sortMemento != null) {
				sortColumn = sortMemento.getString("column");
				sortDirection = "UP".equals(sortMemento.getString("direction")) ? SWT.UP : SWT.DOWN;
			}
		}
		
		if (sortColumn != null) {
			viewer.setSortColumn(sortColumn, sortDirection);
		}
	}
	
	private void makeColumn(String id, Integer width, Boolean initiallyVisible) {
		
		if ("contact".equals(id)) makeContactColumn(width, initiallyVisible);
		else if ("name".equals(id)) makeNameColumn(width, initiallyVisible);
		else if ("email".equals(id)) makeEmailColumn(width, initiallyVisible);
		else if ("mobilephone".equals(id)) makeMobilePhoneColumn(width, initiallyVisible);
		
		// TODO What if we're asked to build a column that doesn't exist?
	}
	
	
	/**
	 * The taskCellDrawListener is added to all the columns displayed in the
	 * table viewer. This listener draws a line through completed contacts.
	 */

	abstract class AbstractCellLabelProvider extends ColumnLabelProvider {
		@Override
		public Color getForeground(Object element) {
			return getTextColour(viewer.getTable().getDisplay(), (Contact)element);
		}

		private Color getTextColour(Device device, Contact contact) {
			return device.getSystemColor(getTextColourConstant(device, contact));
		}
		
		private int getTextColourConstant(Device device, Contact contact) {
			/*if (task.isComplete()) return SWT.COLOR_DARK_GRAY;
			if (isPast(task.getDueDate())) return SWT.COLOR_RED;*/
			return SWT.COLOR_BLACK;
		}
	}
	
	private void makeMobilePhoneColumn(Integer savedWidth, Boolean initiallyVisible) {
		phoneColumn = new TableViewerColumnDefinition(viewer, SWT.LEFT, "mobilephone");
		phoneColumn.setTitle("MobilePhone");
		phoneColumn.setActionText("MobilePhone");
		phoneColumn.setActionDescription("Toggles the MobilePhone column.");
		phoneColumn.setWidth(savedWidth == null ? 250 : savedWidth.intValue());
		phoneColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				Contact contact = (Contact)element;
				if(contact.getMobileNum()==0)
					return  "" ;
				else return String.valueOf(contact.getMobileNum());
			}
		});
		phoneColumn.setComparator(new Comparator<Contact>() {
			public int compare(Contact name0, Contact name1) {
				if (name0.getFullName() == null) {
					if (name1.getFullName() == null) return 0;
					return -1;
				} else if (name1.getFullName() == null) return 1;
				else return name0.getFullName().compareToIgnoreCase(name1.getFullName());
			}			
		});
		phoneColumn.setVisible(initiallyVisible == null ? true : initiallyVisible.booleanValue());
	}
	private void makeContactColumn(Integer savedWidth, Boolean initiallyVisible) {
		final Image image = Activator.getDefault().getImage(Activator.IMAGE_CONTACT);
		final Rectangle imageBounds = image.getBounds();
		
		TableViewerColumnDefinition contactColumn = new TableViewerColumnDefinition(viewer, SWT.NONE, "contact");
		contactColumn.setResizable(false);
		int width = imageBounds.width; // bug in Windows XP https://bugs.eclipse.org/bugs/show_bug.cgi?id=180783
		contactColumn.setMinimumWidth(width);
		contactColumn.setWidth(width);
		contactColumn.addCellDrawListener(new CellDrawListener() { 			
			public void drawCell(CellDrawEvent event) {
				Rectangle bounds = event.getCellBounds();
				Rectangle imageBounds = image.getBounds();
				event.getGC().drawImage(image, bounds.x + (bounds.width - imageBounds.width) / 2, bounds.y + (bounds.height - imageBounds.width) / 2);
			}
		});
		contactColumn.addCellMeasureListener(new CellMeasureListener() {
			public void measureCell(CellMeasureEvent event) {
				event.setHeight(imageBounds.height + 2);				
			}
			
		});
		//contactColumn.addCellDrawListener(taskCellDrawListener);
		contactColumn.setVisible(true);
	}
	
	private void makeEmailColumn(Integer savedWidth, Boolean initiallyVisible) {
		emailColumn = new TableViewerColumnDefinition(viewer, SWT.LEFT, "Email");
		emailColumn.setTitle("Email");
		emailColumn.setActionText("Email");
		emailColumn.setActionDescription("Toggles the Email column.");
		emailColumn.setWidth(savedWidth == null ? 250 : savedWidth.intValue());
		emailColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				Contact contact = (Contact)element;
				return contact.getPrimaryMailAddress();
			}
		});
		emailColumn.setComparator(new Comparator<Contact>() {
			public int compare(Contact name0, Contact name1) {
				if (name0.getFullName() == null) {
					if (name1.getFullName() == null) return 0;
					return -1;
				} else if (name1.getFullName() == null) return 1;
				else return name0.getFullName().compareToIgnoreCase(name1.getFullName());
			}			
		});
		emailColumn.setVisible(initiallyVisible == null ? true : initiallyVisible.booleanValue());
	}
	private void makeNameColumn(Integer savedWidth, Boolean initiallyVisible) {
		nameColumn = new TableViewerColumnDefinition(viewer, SWT.LEFT, "name");
		nameColumn.setTitle("Contact Name");
		nameColumn.setActionText("Contact Name");
		nameColumn.setActionDescription("Toggles the Name column.");
		nameColumn.setWidth(savedWidth == null ? 200 : savedWidth.intValue());
		nameColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				Contact contact = (Contact)element;
				return contact.getFullName();
			}
		});
		nameColumn.setComparator(new Comparator<Contact>() {
			public int compare(Contact name0, Contact name1) {
				if (name0.getPrimaryMailAddress() == null) {
					if (name1.getPrimaryMailAddress() == null) return 0;
					return -1;
				} else if (name1.getPrimaryMailAddress() == null) return 1;
				else return name0.getPrimaryMailAddress().compareToIgnoreCase(name1.getPrimaryMailAddress());
			}			
		});
		nameColumn.setVisible(initiallyVisible == null ? true : initiallyVisible.booleanValue());
	}
	
	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				List<String> receiverList = new ArrayList<String>();
				Contact contact = (Contact)((IStructuredSelection)viewer.getSelection()).getFirstElement();
				receiverList.add(contact.getPrimaryMailAddress());
				new CreateMailAction(receiverList).run();
				
			}
		});
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalToolBar(bars.getToolBarManager());
	}
	
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(refreshAction);
		manager.add(createAction);
		manager.add(deleteAction);
	}
	
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	private boolean isPast(Date date) {
		if (date == null) return false;
		return date.compareTo(new Date()) < 0;
	}

}
