/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.events.ContentChangeEvent;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.listeners.ContentChangeListener;
import org.eclipse.samples.organizer.mails.filters.MailFilter;
import org.eclipse.samples.organizer.mails.filters.MailFilterByFolderID;
import org.eclipse.samples.organizer.main.MailAccount;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.models.Mail;
import org.eclipse.samples.organizer.models.Mail.MailPrivateTag;
import org.eclipse.samples.organizer.models.Mail.Priority;
import org.eclipse.samples.organizer.ui.mail.Activator;
import org.eclipse.samples.organizer.ui.mail.actions.CreateMailAction;
import org.eclipse.samples.organizer.ui.mail.editors.MailEditor;
import org.eclipse.samples.organizer.ui.mail.editors.MailReplyForwardEditor;
import org.eclipse.samples.organizer.ui.mail.editors.MailViewer;
import org.eclipse.samples.organizer.ui.mail.jobs.DeleteMessageJob;
import org.eclipse.samples.organizer.ui.mail.jobs.Pop3GetJob;
import org.eclipse.samples.organizer.ui.mail.main.ExtendedMessage;
import org.eclipse.samples.organizer.ui.mail.util.FileUtil;
import org.eclipse.samples.organizer.ui.util.CellDrawEvent;
import org.eclipse.samples.organizer.ui.util.CellDrawListener;
import org.eclipse.samples.organizer.ui.util.SuperTableViewer;
import org.eclipse.samples.organizer.ui.util.TableViewerColumnDefinition;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.ViewPart;

public class MailView extends ViewPart {

	private SuperTableViewer viewer;

	private SelectionListener attachListener = new SelectionAdapter() {
	};
	private Action openAction;
	private Action retriveAction;
	private Action deleteAction;
	private Action refreshAction;
	private Action makeCreateAction;

	private TableViewerColumnDefinition sentDateColumn;
	private TableViewerColumnDefinition senderColumn;
	private TableViewerColumnDefinition subjectColumn;
	private TableViewerColumnDefinition priorityColumn;
	private TableViewerColumnDefinition sizeColumn;

	private IMemento memento;
	private TableViewerColumnDefinition tagColumn;
	private TableViewerColumnDefinition replyColumn;
	private Action replyAction;
	private Action replyToAllAction;
	private Action forwardAction;
	private Action sashFormRecoverAction;

	private ContentChangeListener contentChangeListener = new ContentChangeListener() {
		public void contentChanged(ContentChangeEvent event) {
			viewer.refresh();
		}
	};

	private SashForm sashForm;

	private Composite previewComposite;

	private Label subjectLabel;

	private Text subjectText;

	private Button attachmentButton;

	private Button previewResizeButtion;

	private Composite cotentComposite;

	class ViewContentProvider implements IStructuredContentProvider,
			ISelectionChangedListener {
		private Object[] mails = new Object[0];

		private PropertyChangeListener propertyChangeListener = new PropertyChangeListener() {

			public void propertyChange(PropertyChangeEvent event) {
				viewer.refresh();
			}
		};

		public ViewContentProvider() {
			hookContentListeners();
		}

		public void inputChanged1(Viewer v, Object oldInput, Object newInput) {
		}

		public void dispose() {
			unhookContentListeners();
			unhookChangeAdapters();
		}

		/**
		 * Hooks a content change listener to the organizer manager so that
		 * whenever content is changed, the receiver gets a chance to update.
		 * Likewise, add property change listener to the MailFilter so that when
		 * it changes, the receiver can update.
		 */
		private void hookContentListeners() {
			OrganizerManager.instance
					.addContentChangeListener(contentChangeListener);
			MailFilterByFolderID.instance
					.addPropertyChangeListener(propertyChangeListener);
		}

		private void unhookContentListeners() {
			OrganizerManager.instance
					.removeContentChangeListener(contentChangeListener);
			MailFilterByFolderID.instance
					.removePropertyChangeListener(propertyChangeListener);
		}

		public Object[] getElements(Object parent) {
			unhookChangeAdapters();
			Job job = new Job("Get mails from current folder!") {
				@Override
				protected IStatus run(IProgressMonitor monitor) {
					fetchMails();
					return Status.OK_STATUS;
				}

				private void fetchMails() {
					try {
						mails = OrganizerManager.instance
								.find(MailFilterByFolderID.instance);
					} catch (BackendException e) {
						e.printStackTrace();
					}
				}
			};
			if (MailManager.getInstance().getCurrentFolder() == null
					|| MailManager.getInstance().isAccountNone()) {
				return mails;
			}
			job.schedule();
			try {
				job.join();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			if (MailManager.getInstance().getCurrentFolder() == null
					|| mails == null) {
				setPartName("Mail");
			} else {
				StringBuilder name = new StringBuilder(MailManager.getInstance().getAccountFromID(MailManager
						.getInstance().getCurrentFolder().getAccountID()).getAccountName());
				name.append(",").append(
						MailManager.getInstance().getCurrentFolder().getFolderName());
				if (mails.length == 0) {
					name.append(",Unread:0/Total:0");
					MailView.this.setPartName(name.toString());
					Activator.getDefault().showMessageOnStatusbar(
							name.toString());
				} else {
					int len = mails.length;
					int j = 0;
					for (int i = 0; i < len; i++) {
						if (!((Mail) mails[i]).isRead()) {
							j++;
						}
					}
					name.append(",Unread:").append(j).append("/Total:").append(
							len);
					setPartName(name.toString());
					Activator.getDefault().showMessageOnStatusbar(
							name.toString());
				}
			}
			hookChangeAdapters();

			return mails;
		}

		private void hookChangeAdapters() {

			if (mails == null)
				return;

			for (Object mail : mails) {
				((Mail) mail).addPropertyChangeListener(propertyChangeListener);
			}
		}

		private void unhookChangeAdapters() {
			if (mails == null)
				return;
			for (Object mail : mails) {
				((Mail) mail)
						.removePropertyChangeListener(propertyChangeListener);
			}
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
             
			 viewer.refresh();
		}

		public void selectionChanged(SelectionChangedEvent event) {

			IStructuredSelection ss = (IStructuredSelection) event
					.getSelection();
			if (ss.getFirstElement() instanceof Mail) {
				final Mail mail = (Mail) ss.getFirstElement();
				subjectLabel.setText("Subject: " + mail.getSubject());
				attachmentButton.removeSelectionListener(attachListener);
				attachmentButton.setEnabled(mail.hasAttachment());

				new Job("Update preview!") {

					@Override
					protected IStatus run(IProgressMonitor monitor) {

						final ExtendedMessage message = new ExtendedMessage();
						message.setAccountID(mail.getAccountID());
						message.getMessageFromFile(mail.getDataFilePath());
						Display.getDefault().syncExec(new Runnable() {

							public void run() {

								for (int i = 0; i < cotentComposite
										.getChildren().length; i++) {
									cotentComposite.getChildren()[i].dispose();
								}

								createCotentsText(cotentComposite, message);

								attachListener = new SelectionAdapter() {
									public void widgetSelected(SelectionEvent e) {
										Menu attachMenu = new Menu(
												attachmentButton);
										for (int j = 0, len = message
												.getAttachmentName().length; j < len; j++) {
											final File file = new File(message
													.getAttachmentName()[j]);
											{
												if (file.exists()) {
													StringBuilder fileItemName = new StringBuilder(
															file.getName());
													fileItemName
															.append("(")
															.append(
																	FileUtil
																			.transferFileSize(file
																					.length()))
															.append(")");
													MenuItem attachmentItem = new MenuItem(
															attachMenu,
															SWT.CASCADE);
													attachmentItem
															.setImage(FileUtil
																	.getIcon(file));
													attachmentItem
															.setText(fileItemName
																	.toString());
													attachmentItem
															.addSelectionListener(new SelectionAdapter() {
																public void widgetSelected(
																		SelectionEvent e) {
																	Program
																			.launch(file
																					.getAbsolutePath());
																}

															});
												}
											}
										}
										if (!attachMenu.getVisible()) {
											attachMenu.setVisible(true);
										}
									}
								};
								attachmentButton
										.addSelectionListener(attachListener);

							}
						});
						return Status.OK_STATUS;

					}
				}.schedule();

			}
		}
	}

	private void createCotentsText(Composite parent, ExtendedMessage _message) {
		if (_message.getMessageFormat() == Mail.MailFormat.PLAIN) {
			Text detailsText = new Text(parent, SWT.MULTI | SWT.WRAP
					| SWT.V_SCROLL | SWT.BORDER);
			detailsText.setSize(parent.getSize());
			detailsText.setText(_message.getPlainTextFromBodyPart());

		} else {
			Browser browser;
			try {
				browser = new Browser(parent, SWT.MULTI | SWT.WRAP
						| SWT.V_SCROLL | SWT.BORDER);
			} catch (SWTError e) {
				System.out.println("Could not instantiate Browser: "
						+ e.getMessage());
				return;
			}
			if (_message.getHTMLFromBodyPart() != null)
				browser.setText(_message.getHTMLFromBodyPart());
			browser.setSize(parent.getSize());
		}
		/*
		 * shell.open(); while (!shell.isDisposed()) { if
		 * (!display.readAndDispatch()) display.sleep(); } display.dispose(); }
		 */
	}

	public void createPartControl(Composite parent) {

		sashForm = new SashForm(parent, SWT.VERTICAL);
		viewer = new SuperTableViewer(sashForm, SWT.FULL_SELECTION | SWT.MULTI
				| SWT.H_SCROLL | SWT.V_SCROLL);

		previewComposite = new Composite(sashForm, SWT.BORDER);
		createPreview(previewComposite);

		// Bind the table to the s.
		Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		ViewContentProvider vcp = new ViewContentProvider();
		viewer.setContentProvider(vcp);
		getSite().setSelectionProvider(viewer);
		viewer.addSelectionChangedListener(vcp);
		makeActions();

		hookContextMenu();
		hookDoubleClickAction();
		contributeToActionBars();

		// Add initial columns
		makeColumns();

		hookListeners();
		viewer.setInput(getViewSite());
		MailManager.getInstance().addContentChangeListener(
				contentChangeListener);

	}

	private void createPreview(Composite _previewComposite) {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		gridLayout.horizontalSpacing = 5;
		_previewComposite.setLayout(gridLayout);

		subjectLabel = new Label(_previewComposite, SWT.NONE);
		subjectLabel.setText("Mail preview area");

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;

		subjectText = new Text(_previewComposite, SWT.READ_ONLY);
		subjectText.setLayoutData(gridData);

		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = false;
		attachmentButton = new Button(_previewComposite, SWT.NONE);
		attachmentButton.setImage(Activator.getDefault().getImage(
				Activator.IMAGE_MAIL_ATTACHMENT));
		attachmentButton.setLayoutData(gridData1);
		attachmentButton.setEnabled(false);
		previewResizeButtion = new Button(_previewComposite, SWT.NONE);
		previewResizeButtion.setImage(Activator.getDefault().getImage(
				Activator.DOWN_IMAGE_PATH));
		previewResizeButtion.setLayoutData(gridData1);
		previewResizeButtion.addMouseListener(new MouseAdapter() {
			public void mouseDown(MouseEvent e) {
				sashForm.setWeights(new int[] { 1, 0 });
			}
		});

		GridData gridData3 = new GridData();
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.verticalAlignment = GridData.FILL;
		gridData3.horizontalSpan = 5;
		gridData3.grabExcessVerticalSpace = true;
		cotentComposite = new Composite(_previewComposite, SWT.NONE);
		cotentComposite.setLayoutData(gridData3);
		cotentComposite.setLayout(new FillLayout());
		/*
		 * Text text = new Text(cotentComposite,SWT.BORDER); GridData gridData4 =
		 * new GridData(); gridData4.grabExcessHorizontalSpace = true;
		 * gridData4.grabExcessVerticalSpace = true;
		 * //gridData4.horizontalAlignment = GridData.FILL;
		 * //gridData4.verticalAlignment = GridData.FILL; text.setText("test");
		 * text.setVisible(true); text.setLayoutData(gridData4);
		 */

	}

	@Override
	public void saveState(IMemento memento) {
		// Save the column state.
		IMemento columnsMemento = memento.createChild("columns");
		for (TableViewerColumnDefinition column : viewer.getColumnDefinitions()) {
			IMemento columnMemento = columnsMemento.createChild("column");
			columnMemento.putString("id", column.getId());
			columnMemento.putInteger("width", column.getWidth());
			columnMemento.putString("visible", String.valueOf(column
					.isVisible()));
		}
		// Save the sort state.
		IMemento sortMemento = memento.createChild("sort");
		sortMemento.putString("column", viewer.getSortColumnId());
		sortMemento.putString("direction",
				viewer.getSortDirection() == SWT.UP ? "UP" : "DOWN");
	}

	@Override
	public void init(IViewSite site, IMemento memento) throws PartInitException {
		this.memento = memento;
		super.init(site, memento);

	}

	private void hookListeners() {
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			public void selectionChanged(SelectionChangedEvent event) {
				updateActions();

			}
		});
	}

	private void updateActions() {
		boolean isDraftSelected = false;
		IStructuredSelection selection = (IStructuredSelection) viewer
				.getSelection();
		boolean isAnythingSelected = selection.size() > 0;
		if (isAnythingSelected)
			isDraftSelected = ((Mail) selection.getFirstElement())
					.getMailflag() == Mail.MailFlag.DRAFT;
		deleteAction.setEnabled(isAnythingSelected);
		openAction.setEnabled(isAnythingSelected);
		makeCreateAction.setEnabled(true);
		retriveAction.setEnabled(true);
		replyAction.setEnabled(isAnythingSelected && !isDraftSelected);
		replyToAllAction.setEnabled(isAnythingSelected && !isDraftSelected);
		forwardAction.setEnabled(isAnythingSelected && !isDraftSelected);

	}

	public void dispose() {
		super.dispose();
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void fillContextMenu(IMenuManager manager) {

		manager.add(makeCreateAction);
		manager.add(retriveAction);
		manager.add(openAction);
		manager.add(replyAction);
		manager.add(replyToAllAction);
		manager.add(forwardAction);
		manager.add(new Separator());
		manager.add(refreshAction);
		manager.add(deleteAction);
		manager.add(new Separator());
		// Create and populate a submenu that the user can use
		// to show/hide columns.
		IMenuManager columnsMenu = new MenuManager("Columns");
		// TODO Fill the columns menu
		viewer.fillColumnsMenu(columnsMenu);
		manager.add(columnsMenu);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void makeActions() {
		makeCreateMailAction();
		makeOpenAction();
		makeReplyAction();
		makeReplyToAllAction();
		makeForwardAction();
		makeRetriveAction();

		makeRefreshAction();

		makeDeleteAction();
		makeSashFormRecoverAction();
		updateActions();
	}

	private void makeSashFormRecoverAction() {
		sashFormRecoverAction = new Action() {
			public void run() {
				sashForm.setWeights(new int[] { 1, 1 });
			}
		};
		sashFormRecoverAction.setText("Recover perview area");
		sashFormRecoverAction.setToolTipText("Recover perview area");
		sashFormRecoverAction.setImageDescriptor(ImageDescriptor
				.createFromImage(Activator.getDefault().getImage(
						Activator.UP_IMAGE_PATH)));
	}

	private void makeForwardAction() {
		forwardAction = new Action() {
			public void run() {

				IWorkbenchPage page = getSite().getPage();
				Mail mail = (Mail) ((IStructuredSelection) viewer
						.getSelection()).getFirstElement();
				ExtendedMessage message = new ExtendedMessage();
				message.setAccountID(mail.getAccountID());
				message.setMessageoperation(ExtendedMessage.MessageOperation.Forward);
		        new MailReplyForwardEditor(mail, message).show();
			}
		};
		forwardAction.setText("Forward");
	}

	private void makeReplyToAllAction() {
		replyToAllAction = new Action() {
         public void run(){
        	// message.setMessageoperation(ExtendedMessage.MessageOperation.Forward);
          //   new MailReplyForwardEditor(mail, message).show();
         }
		};
		replyToAllAction.setText("ReplyToAll");

	}

	private void makeReplyAction() {
		replyAction = new Action() {
			public void run() {
				IWorkbenchPage page = getSite().getPage();
				Mail mail = (Mail) ((IStructuredSelection) viewer
						.getSelection()).getFirstElement();
				ExtendedMessage message = new ExtendedMessage();
				message.setAccountID(mail.getAccountID());
				message.getMessageFromFile(mail.getDataFilePath());
				if (mail.getMailflag() != Mail.MailFlag.DRAFT) {
					message.setMessageoperation(ExtendedMessage.MessageOperation.Reply);
		            new MailReplyForwardEditor(mail, message).show();
				}

			}
		};
		replyAction.setText("Reply");
	}

	private void makeOpenAction() {

		openAction = new Action() {
			@Override
			public void run() {
				// IWorkbenchPage page = getSite().getPage();
				final Mail mail = (Mail) ((IStructuredSelection) viewer
						.getSelection()).getFirstElement();
				final ExtendedMessage message = new ExtendedMessage();
				message.setAccountID(mail.getAccountID());
				message.getMessageFromFile(mail.getDataFilePath());
				if (mail.getMailflag() == Mail.MailFlag.DRAFT) {
					message
							.setMessageoperation(ExtendedMessage.MessageOperation.Edited);

					Display.getDefault().asyncExec(new Runnable() {
						public void run() {
							MailEditor mv = new MailEditor(mail, message);
							mv.show();
						}
					});
				} else {
					if (!mail.isRead()) {
						mail.setReadToDatabase(1);
					}
					Display.getDefault().asyncExec(new Runnable() {
						public void run() {
							MailViewer mv = new MailViewer(mail, message);
							mv.show();
						}
					});

				}
			}
		};
		openAction.setText("Open");
		openAction.setToolTipText("Open/edit this message");
	}

	private void makeRefreshAction() {
		refreshAction = new Action() {
			public void run() {
				viewer.refresh();
			}
		};
		refreshAction.setText("Refresh");
		refreshAction.setToolTipText("Refreshes the mail list.");
		refreshAction.setImageDescriptor(ImageDescriptor
				.createFromImage(Activator.getDefault().getImage(
						Activator.IMAGE_MAIL_REFRESH)));
	}

	private void makeRetriveAction() {
		retriveAction = new Action() {
			@Override
			public void run() {
				MailAccount account = MailManager.getInstance()
						.getCurrentaccount();

				new Pop3GetJob(account.getAccountID()).schedule();

			}
		};
		retriveAction.setText("Receive");
		retriveAction.setToolTipText("Receive new mails");
		retriveAction.setImageDescriptor(ImageDescriptor
				.createFromImage(Activator.getDefault().getImage(
						Activator.IMAGE_MAIL_RETRIEVE)));
	}

	private void makeDeleteAction() {
		deleteAction = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				List<Mail> s = (List<Mail>) ((IStructuredSelection) selection)
						.toList();
				if (s == null)
					return;
				if (s.size() == 0)
					return;
				new DeleteMessageJob(s).schedule();
				viewer.refresh();
			}
		};
		deleteAction.setText("Delete");
		deleteAction.setToolTipText("Delete the currently selected message(s)");
		deleteAction
				.setImageDescriptor(getSharedImage(ISharedImages.IMG_TOOL_DELETE));
		deleteAction
				.setDisabledImageDescriptor(getSharedImage(ISharedImages.IMG_TOOL_DELETE_DISABLED));
		deleteAction.setActionDefinitionId(ActionFactory.DELETE.getId());
	}

	private void makeCreateMailAction() {

		makeCreateAction = new CreateMailAction(new ArrayList<String>());
	}

	private ImageDescriptor getSharedImage(String id) {
		return PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
				id);
	}

	private void makeColumns() {
		makeMessageColumn();
		makeAttachmentColumn();
		makeReplyColumn();
		makePriorityColumn();
		makeSenderColumn();
		makeSubjectColumn();
		makeSentDateColumn();
		makeSizeColumn();
		makeTagColumn();
	}

	abstract class AbstractCellLabelProvider extends ColumnLabelProvider {

		public Color getForeground(Object element) {
			return getTextColour(viewer.getTable().getDisplay(), (Mail) element);
		}

		private Color getTextColour(Device device, Mail mail) {
			return device.getSystemColor(getTextColourConstant(device, mail));
		}

		private int getTextColourConstant(Device device, Mail mail) {

			Mail.MailPrivateTag tag = mail.getMailprivatetag();
			if (tag == MailPrivateTag.NONE)
				return SWT.COLOR_BLACK;
			else if (tag == MailPrivateTag.IMPORTANT)
				return SWT.COLOR_RED;
			else if (tag == MailPrivateTag.HOBBY)
				return SWT.COLOR_DARK_YELLOW;
			else if (tag == MailPrivateTag.FUNNY)
				return SWT.COLOR_BLUE;
			else if (tag == MailPrivateTag.FRIEND)
				return SWT.COLOR_DARK_MAGENTA;
			else if (tag == MailPrivateTag.FAMILY)
				return SWT.COLOR_GREEN;
			else if (tag == MailPrivateTag.COMPANY)
				return SWT.COLOR_DARK_RED;
			else if (tag == MailPrivateTag.BUSINESS)
				return SWT.COLOR_DARK_CYAN;
			else if (tag == MailPrivateTag.TECHONOLOGY)
				return SWT.COLOR_DARK_GREEN;
			else
				return SWT.COLOR_BLACK;
		}
	}

	private void makeMessageColumn() {
		final Image headerImage = getImage(Activator.IMAGE_MAIL16);
		final Image messageImage = getImage(Activator.IMAGE_MAIL_OPEN);
		final Image draftImage = getImage(Activator.IMAGE_MAIL_DRAFT);

		final Rectangle imageBounds = headerImage.getBounds();

		TableViewerColumnDefinition messageColumn = new TableViewerColumnDefinition(
				viewer, SWT.NONE, "message");
		messageColumn.setImage(headerImage);
		messageColumn.setResizable(false);
		int width = imageBounds.width * 3 / 2; // bug in Windows XP
		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=180783
		messageColumn.setMinimumWidth(width);
		messageColumn.setWidth(width);
		messageColumn.setLabelProvider(new ColumnLabelProvider() {
			// @Override
			/*
			 * public Image getImage(Object element) { Mail mail =
			 * (Mail)element; if (mail.isRead()) return headerImage; return
			 * messageImage; }
			 */
			@Override
			public String getText(Object element) {
				return "";
			}
		});
		messageColumn.setEditingSupport(new EditingSupport(viewer) {
			@Override
			protected boolean canEdit(Object element) {
				return true;
			}

			@Override
			protected CellEditor getCellEditor(Object element) {
				return new CheckboxCellEditor(viewer.getTable());
			}

			@Override
			protected Object getValue(Object element) {
				return ((Mail) element).isRead();

			}

			@Override
			protected void setValue(Object element, Object value) {

				if (((Boolean) value).booleanValue())
					((Mail) element).setReadToDatabase(1);
				else
					((Mail) element).setReadToDatabase(0);
				viewer.refresh();
			}

		});
		messageColumn.addCellDrawListener(new CellDrawListener() {
			public void drawCell(CellDrawEvent event) {
				Image image = getImage((Mail) event.getObject());
				Rectangle imageBounds = image.getBounds();
				Rectangle bounds = event.getCellBounds();
				event.getGC().drawImage(image,
						bounds.x + (bounds.width - imageBounds.width) / 2,
						bounds.y + (bounds.height - imageBounds.width) / 2);
			}

			private Image getImage(Mail mail) {
				if (mail.getMailflag() == Mail.MailFlag.DRAFT)
					return draftImage;
				return mail.isRead() ? messageImage : headerImage;
			}
		});
		messageColumn.setComparator(new Comparator<Mail>() {
			public int compare(Mail mail0, Mail mail1) {
				if (mail0.getRead() > 0 || mail1.getRead() > 0)
					return 1;
				else if (mail0.getRead() == 0 || mail1.getRead() == 0)
					return -1;
				return 0;
			}
		});
		// Column.addCellDrawListener(CellDrawListener);
		messageColumn.setVisible(true);
	}

	private void makeAttachmentColumn() {
		final Image attachmentImage = getImage(Activator.IMAGE_MAIL_ATTACHMENT);

		final Rectangle imageBounds = attachmentImage.getBounds();

		TableViewerColumnDefinition attachmentColumn = new TableViewerColumnDefinition(
				viewer, SWT.NONE, "message");
		attachmentColumn.setResizable(false);
		// attachmentColumn.setImage(attachmentImage);
		int width = imageBounds.width;
		attachmentColumn.setMinimumWidth(width);
		attachmentColumn.setWidth(width);
		attachmentColumn.setLabelProvider(new ColumnLabelProvider() {
			// @Override
			public Image getImage(Object element) {
				Mail mail = (Mail) element;
				if (mail.hasAttachment())
					return attachmentImage;
				return null;
			}

			@Override
			public String getText(Object element) {
				return "";
			}
		});

		attachmentColumn.setComparator(new Comparator<Mail>() {
			public int compare(Mail mail0, Mail mail1) {
				if (mail0.hasAttachment() && !mail1.hasAttachment()) {
					return 1;
				} else if (mail1.hasAttachment() && !mail0.hasAttachment())
					return -1;
				return 0;
			}
		});
		attachmentColumn.setVisible(true);
	}

	private void makeReplyColumn() {
		final Image replyforwardImage = Activator.getDefault().getImage(
				Activator.IMAGE_REPLYSYMBOL);
		final Rectangle imageBounds = replyforwardImage.getBounds();

		replyColumn = new TableViewerColumnDefinition(viewer, SWT.CENTER,
				"Reply&Forward");
		replyColumn.setResizable(false);
		replyColumn.setTitle("R");
		replyColumn.setActionText("&Replied Forwareded");
		replyColumn.setActionDescription("Tag for Replied or Forwarded mail");
		int width = imageBounds.width; // bug in Windows XP
		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=180783
		replyColumn.setMinimumWidth(width);
		replyColumn.setWidth(width);
		replyColumn.setLabelProvider(new ColumnLabelProvider() {
			public Image getImage(Object element) {
				Mail mail = (Mail) element;
				if (mail.getMailflag() == Mail.MailFlag.ANSWERED)
					return replyforwardImage;
				return null;
			}

			@Override
			public String getText(Object element) {
				return "";
			}
		});
/*
 * replyColumn.setEditingSupport(new EditingSupport(viewer) { @Override
 * protected boolean canEdit(Object element) { return true; }
 * 
 * @Override protected CellEditor getCellEditor(Object element) { return new
 * CheckboxCellEditor(viewer.getTable()); }
 * 
 * @Override protected Object getValue(Object element) { return
 * ((Mail)element).getMailflag(); }
 * 
 * @Override protected void setValue(Object element, Object value) {
 * 
 * if(((Mail)value).getMailflag()==MailFlag.ANSWERED)
 * ((Mail)element).setMailflagToDatabase(MailFlag.RECENT); else
 * ((Mail)element).setMailflagToDatabase(MailFlag.ANSWERED); }
 * 
 * });
 */
/*
 * replyColumn.addCellDrawListener(new CellDrawListener() { public void
 * drawCell(CellDrawEvent event) { Mail mail = (Mail)event.getObject(); Image
 * image = getImage(mail.getMailflag()); if(image==null){
 * event.getGC().drawText("", 0, 0); } Rectangle imageBounds =
 * image.getBounds(); Rectangle bounds = event.getCellBounds();
 * event.getGC().drawImage(image, bounds.x + (bounds.width - imageBounds.width) /
 * 2, bounds.y + (bounds.height - imageBounds.width) / 2); }
 * 
 * private Image getImage(Mail.MailFlag flag) { if(flag==Mail.MailFlag.ANSWERED)
 * return replyforwardImage; return null; } }); replyColumn.setComparator(new
 * Comparator<Mail>() { public int compare(Mail mail0, Mail mail1) {
 * if(mail0.getMailflag()==MailFlag.ANSWERED&&mail1.getMailflag()==MailFlag.ANSWERED)
 * return 1; return -1; } });
 */

	}

	private Image getImage(String path) {
		return Activator.getDefault().getImage(path);
	}

	private void makeSentDateColumn() {
		sentDateColumn = new TableViewerColumnDefinition(viewer, SWT.RIGHT,
				"date");
		sentDateColumn.setTitle("Sent");
		sentDateColumn.setActionText("Sent Date");
		sentDateColumn.setActionDescription("Toggles the Sent Date column");
		sentDateColumn.setWidth(120);

		sentDateColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				Mail mail = (Mail) element;
				return mail.getDate() != null ? mail.getDate() : "";
			}
		});
		sentDateColumn.setComparator(new Comparator<Mail>() {
			public int compare(Mail mail0, Mail mail1) {
				if (mail0.getDate() == null) {
					if (mail1.getDate() == null)
						return 0;
					return -1;
				} else if (mail1.getDate() == null)
					return 1;
				else
					return mail0.getDate().compareTo(mail1.getDate());
			}
		});

// sentDateColumn.setComparator(new Comparator<>() {
// public int compare( 0, 1) {
// if (0.getDueDate() == null) {
// if (1.getDueDate() == null) return 0;
// return -1;
// } else if (1.getDueDate() == null) return 1;
// else return 0.getDueDate().compareTo(1.getDueDate());
// }
// });
		sentDateColumn.setVisible(true);
	}

	private void makeSubjectColumn() {
		subjectColumn = new TableViewerColumnDefinition(viewer, SWT.LEFT,
				"subject");
		subjectColumn.setTitle("Subject");
		subjectColumn.setActionText("Subject");
		subjectColumn.setActionDescription("Toggles the Subject column.");
		subjectColumn.setWidth(250);
		subjectColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				Mail mail = (Mail) element;
				if (mail.getSubject() != null)
					return mail.getSubject();
				return "";
			}
		});


		subjectColumn.setComparator(new Comparator<Mail>() {
			public int compare(Mail mail0, Mail mail1) {
				if (mail0.getSubject() == null) {
					if (mail1.getSubject() == null)
						return 0;
					return -1;
				} else if (mail1.getSubject() == null)
					return 1;
				else
					return mail0.getSubject().compareToIgnoreCase(
							mail1.getSubject());
			}
		});
		subjectColumn.setVisible(true);
	}

	private void makeSizeColumn() {
		sizeColumn = new TableViewerColumnDefinition(viewer, SWT.RIGHT, "Size");
		sizeColumn.setTitle("Size");
		sizeColumn.setActionText("Size");
		sizeColumn.setActionDescription("Toggles the Subject column.");
		sizeColumn.setMinimumWidth(50);
		sizeColumn.setWidth(50);
		sizeColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				Mail mail = (Mail) element;
				if (mail.getSize() > 0 && mail.getSize() < 1024)
					return "1K";
				else if (mail.getSize() > 1024 && mail.getSize() < 1024 * 1024)
					return String.valueOf((int) (mail.getSize() / 1024)) + "K";
				else if (mail.getSize() > 1024 * 1024)
					return String
							.valueOf((int) (mail.getSize() / (1024 * 1024)))
							+ "M";
				return "0K";
			}
		});

		sizeColumn.setComparator(new Comparator<Mail>() {
			public int compare(Mail mail0, Mail mail1) {
				if (mail0.getSize() == (mail1.getSize()))
					return 0;
				return (mail0.getSize() < (mail1.getSize())) ? 1 : -1;
			}
		});

		sizeColumn.setVisible(true);

	}

	private void makeTagColumn() {
		tagColumn = new TableViewerColumnDefinition(viewer, SWT.LEFT, "Tag");
		tagColumn.setTitle("Private Tags");
		tagColumn.setActionText("Private Tags");
		tagColumn.setActionDescription("Tags for your private divisions.");
		tagColumn.setWidth(150);
		/*
		 * tagColumn.setLabelProvider(new AbstractCellLabelProvider() {
		 * @Override public String getText(Object element) { Mail mail =
		 * (Mail)element; if(mail.getMailprivatetag()!=Mail.MailPrivateTag.NONE)
		 * return mail.getMailprivatetag().name(); return ""; } });
		 */
		tagColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				MailPrivateTag tag = ((Mail) element).getMailprivatetag();
				if (tag == MailPrivateTag.NONE)
					return "";
				else if (tag == MailPrivateTag.IMPORTANT)
					return "IMPORTANT";
				else if (tag == MailPrivateTag.HOBBY)
					return "HOBBY";
				else if (tag == MailPrivateTag.FUNNY)
					return "FUNNY";
				else if (tag == MailPrivateTag.FRIEND)
					return "FRIEND";
				else if (tag == MailPrivateTag.FAMILY)
					return "FAMILY";
				else if (tag == MailPrivateTag.COMPANY)
					return "COMPANY";
				else if (tag == MailPrivateTag.BUSINESS)
					return "BUSINESS";
				else if (tag == MailPrivateTag.TECHONOLOGY)
					return "TECHONOLOGY";
				else
					return "";
			}
		});
		tagColumn.setEditingSupport(new EditingSupport(viewer) {
			private String[] tags = new String[] { "None", "IMPORTANT",
					"COMPANY", "FRIEND", "FUNNY", "HOBBY", "FAMILY",
					"BUSINESS", "TECHONOLOGY" };

			@Override
			protected boolean canEdit(Object element) {
				return true;
			}

			@Override
			protected CellEditor getCellEditor(Object element) {
				return new ComboBoxCellEditor(viewer.getTable(), tags);
			}

			@Override
			protected Object getValue(Object element) {
				MailPrivateTag tag = ((Mail) element).getMailprivatetag();
				if (tag == MailPrivateTag.NONE)
					return 0;
				else if (tag == MailPrivateTag.IMPORTANT)
					return 1;
				else if (tag == MailPrivateTag.HOBBY)
					return 2;
				else if (tag == MailPrivateTag.FUNNY)
					return 3;
				else if (tag == MailPrivateTag.FRIEND)
					return 4;
				else if (tag == MailPrivateTag.FAMILY)
					return 5;
				else if (tag == MailPrivateTag.COMPANY)
					return 6;
				else if (tag == MailPrivateTag.BUSINESS)
					return 7;
				else if (tag == MailPrivateTag.TECHONOLOGY)
					return 8;
				else
					return null;
			}

			@Override
			protected void setValue(Object element, Object value) {
				MailPrivateTag tag = null;
				switch (((Integer) value).intValue()) {
				case 0:
					tag = MailPrivateTag.NONE;
					break;
				case 1:
					tag = MailPrivateTag.IMPORTANT;
					break;
				case 2:
					tag = MailPrivateTag.HOBBY;
					break;
				case 3:
					tag = MailPrivateTag.FUNNY;
					break;
				case 4:
					tag = MailPrivateTag.FRIEND;
					break;
				case 5:
					tag = MailPrivateTag.FAMILY;
					break;
				case 6:
					tag = MailPrivateTag.COMPANY;
					break;
				case 7:
					tag = MailPrivateTag.BUSINESS;
					break;
				case 8:
					tag = MailPrivateTag.TECHONOLOGY;
					break;
				}
				((Mail) element).setMailTagToDatabase(tag);
			}
		});
/*
 * tagColumn.addCellDrawListener(new CellDrawListener() { public void
 * drawCell(CellDrawEvent event) { Mail mail = (Mail) event.getObject(); String
 * s = mail.getMailprivatetag().name(); if (s.equalsIgnoreCase("None")) s = "";
 * Rectangle bounds = event.getCellBounds(); //
 * event.getGC().setForeground(getLineColour(event.getGC().getDevice(), //
 * mail.getMailprivatetag())); Mail.MailPrivateTag tag =
 * mail.getMailprivatetag(); int color; if (tag == MailPrivateTag.NONE) color =
 * SWT.COLOR_BLACK; else if (tag == MailPrivateTag.IMPORTANT) color =
 * SWT.COLOR_RED; else if (tag == MailPrivateTag.HOBBY) color =
 * SWT.COLOR_DARK_YELLOW; else if (tag == MailPrivateTag.FUNNY) color =
 * SWT.COLOR_BLUE; else if (tag == MailPrivateTag.FRIEND) color =
 * SWT.COLOR_DARK_MAGENTA; else if (tag == MailPrivateTag.FAMILY) color =
 * SWT.COLOR_GREEN; else if (tag == MailPrivateTag.COMPANY) color =
 * SWT.COLOR_DARK_RED; else if (tag == MailPrivateTag.BUSINESS) color =
 * SWT.COLOR_DARK_CYAN; else if (tag == MailPrivateTag.TECHONOLOGY) color =
 * SWT.COLOR_DARK_BLUE; else color = SWT.COLOR_BLACK;
 * event.getGC().setForeground(3); event.getGC().drawText(s, bounds.x,
 * bounds.y); }
 * 
 * private Color getLineColour(Device device,MailPrivateTag mpt) { if (mpt ==
 * MailPrivateTag.IMPORTANT) return device.getSystemColor(SWT.COLOR_DARK_RED);
 * else if (mpt == MailPrivateTag.HOBBY) return
 * device.getSystemColor(SWT.COLOR_DARK_GREEN); else if (mpt ==
 * MailPrivateTag.FUNNY) return device.getSystemColor(SWT.COLOR_GRAY); else if
 * (mpt == MailPrivateTag.FRIEND) return device.getSystemColor(SWT.COLOR_CYAN);
 * else if (mpt == MailPrivateTag.FAMILY) return
 * device.getSystemColor(SWT.COLOR_DARK_YELLOW); else if (mpt ==
 * MailPrivateTag.COMPANY) return device.getSystemColor(SWT.COLOR_BLACK); else
 * if (mpt == MailPrivateTag.BUSINESS) return
 * device.getSystemColor(SWT.COLOR_DARK_MAGENTA); else return null; }
 * 
 * 
 * });
 */

		tagColumn.setComparator(new Comparator<Mail>() {
			public int compare(Mail mail0, Mail mail1) {
				return mail0.getMailprivatetag().compareTo(
						mail1.getMailprivatetag());
			}
		});

		tagColumn.setVisible(true);

	}

	private void makeSenderColumn() {
		senderColumn = new TableViewerColumnDefinition(viewer, SWT.LEFT,
				"sender");
		senderColumn.setTitle("Sender");
		senderColumn.setActionText("Sender");
		senderColumn.setActionDescription("Toggles the Sender column.");
		senderColumn.setWidth(250);
		senderColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				Mail mail = (Mail) element;
				if (mail.getSubject() != null)
					return mail.getSender();

				return "";
			}
		});
		senderColumn.setComparator(new Comparator<Mail>() {
			public int compare(Mail mail0, Mail mail1) {
				if (mail0.getSender() == null) {
					if (mail1.getSender() == null)
						return 0;
					return -1;
				} else if (mail1.getSender() == null)
					return 1;
				else
					return mail0.getSender().compareToIgnoreCase(
							mail1.getSender());
			}
		});
// senderColumn.setComparator(new Comparator<>() {
// public int compare( 0, 1) {
// return 0.getSubject().compareToIgnoreCase(1.getSubject());
// }
// });
		senderColumn.setVisible(true);
	}

	private void makePriorityColumn() {
		final Image low = Activator.getDefault().getImage(Activator.IMAGE_LOW);
		final Image normal = Activator.getDefault().getImage(
				Activator.IMAGE_NORMAL);
		final Image high = Activator.getDefault()
				.getImage(Activator.IMAGE_HIGH);

		priorityColumn = new TableViewerColumnDefinition(viewer, SWT.CENTER,
				"priority");
		// TODO Need fancier graphic in the header.
		priorityColumn.setTitle("!");
		priorityColumn.setActionText("Priority");
		priorityColumn.setActionDescription("Toggles the Priority column");
		priorityColumn.setWidth(low.getBounds().width * 2);
		priorityColumn.setMinimumWidth(low.getBounds().width * 2);
		priorityColumn.setResizable(false);

		priorityColumn.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return "";
			}
			/*
			 * public Image getImage(Object element) { if
			 * (((Mail)element).getPriority() == Priority.LOW) return low; else
			 * if (((Mail)element).getPriority() == Priority.NORMAL) return
			 * normal; else if (((Mail)element).getPriority() == Priority.HIGH)
			 * return high; else return null; }
			 */
		});
		priorityColumn.setEditingSupport(new EditingSupport(viewer) {
			private String[] priorities = new String[] { "Low", "Normal",
					"High" };

			@Override
			protected boolean canEdit(Object element) {
				return true;
			}

			@Override
			protected CellEditor getCellEditor(Object element) {
				return new ComboBoxCellEditor(viewer.getTable(), priorities);
			}

			@Override
			protected Object getValue(Object element) {
				Priority priority = ((Mail) element).getPriority();
				if (priority == Priority.LOW)
					return 0;
				else if (priority == Priority.NORMAL)
					return 1;
				else if (priority == Priority.HIGH)
					return 2;
				else
					return null;
			}

			@Override
			protected void setValue(Object element, Object value) {
				Priority priority = null;
				switch (((Integer) value).intValue()) {
				case 0:
					priority = Priority.LOW;
					break;
				case 1:
					priority = Priority.NORMAL;
					break;
				case 2:
					priority = Priority.HIGH;
					break;
				}
				((Mail) element).setPriority(priority);
			}
		});
		priorityColumn.addCellDrawListener(new CellDrawListener() {
			public void drawCell(CellDrawEvent event) {
				Mail mail = (Mail) event.getObject();
				Image image = getImage(mail.getPriority());
				Rectangle imageBounds = image.getBounds();
				Rectangle bounds = event.getCellBounds();
				event.getGC().drawImage(image,
						bounds.x + (bounds.width - imageBounds.width) / 2,
						bounds.y + (bounds.height - imageBounds.width) / 2);
			}

			public Image getImage(Priority priority) {
				if (priority == Priority.LOW)
					return low;
				else if (priority == Priority.NORMAL)
					return normal;
				else if (priority == Priority.HIGH)
					return high;
				else
					return null;
			}
		});

		priorityColumn.setComparator(new Comparator<Mail>() {
			public int compare(Mail mail0, Mail mail1) {
				return mail0.getPriority().compareTo(mail1.getPriority());
			}
		});
	}

	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (openAction.isEnabled())
					openAction.run();
			}
		});
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(makeCreateAction);
		/*
		 * manager.add(replyAction); manager.add(replyToAllAction);
		 * manager.add(forwardAction);
		 */
		manager.add(retriveAction);

		manager.add(deleteAction);
		manager.add(sashFormRecoverAction);

	}

	public void setFocus() {
		viewer.getControl().setFocus();
		viewer.refresh();
	}

	public SuperTableViewer getviewer() {
		return viewer;
	}

}
