/*******************************************************************************
 * Copyright (c) 2006, 2007 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.task.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.Visitor;
import org.eclipse.samples.organizer.events.ContentChangeEvent;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.listeners.ContentChangeListener;
import org.eclipse.samples.organizer.models.Task;
import org.eclipse.samples.organizer.models.Task.Priority;
import org.eclipse.samples.organizer.tasks.filters.TaskFilter;
import org.eclipse.samples.organizer.ui.datepicker.DateCellEditor;
import org.eclipse.samples.organizer.ui.task.Activator;
import org.eclipse.samples.organizer.ui.task.dnd.TaskDropTarget;
import org.eclipse.samples.organizer.ui.task.editors.TaskEditorInput;
import org.eclipse.samples.organizer.ui.task.wizards.ImportTasksWizard;
import org.eclipse.samples.organizer.ui.util.CellDrawEvent;
import org.eclipse.samples.organizer.ui.util.CellDrawListener;
import org.eclipse.samples.organizer.ui.util.CellMeasureEvent;
import org.eclipse.samples.organizer.ui.util.CellMeasureListener;
import org.eclipse.samples.organizer.ui.util.SuperTableViewer;
import org.eclipse.samples.organizer.ui.util.TableViewerColumnDefinition;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.progress.WorkbenchJob;

import com.ibm.icu.text.DateFormat;

public class TasksView extends ViewPart {
	private SuperTableViewer viewer;

	private Action openAction;
	private Action deleteAction;
	private Action createAction;
	private Action refreshAction;
	
	private TableViewerColumnDefinition dueDateColumn;
	private TableViewerColumnDefinition reminderDateColumn;
	private TableViewerColumnDefinition completeColumn;
	private TableViewerColumnDefinition subjectColumn;
	private TableViewerColumnDefinition priorityColumn;
			
	private IMemento memento;
		
	class ViewContentProvider implements IStructuredContentProvider {
		private List<Task> tasks = new ArrayList<Task>();
		
		private ContentChangeListener contentChangeListener = new ContentChangeListener() {
			public void contentChanged(ContentChangeEvent event) {
				viewer.refresh();
			}
		};
		
		private PropertyChangeListener propertyChangeListener = new PropertyChangeListener() {
			/*
			 * We go a little overboard here. When a property on one of the 
			 * tasks changes, update the entire table. The change may affect
			 * the sort order.
			 * TODO Is there a way to update just the affected row and force a resort?
			 */
			public void propertyChange(PropertyChangeEvent event) {
				viewer.refresh();
			}
		};
		
		public ViewContentProvider() {
			hookContentListeners();
		}

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}

		public void dispose() {
			unhookContentListeners();
			unhookChangeListener();
		}
		
		/**
		 * Hooks a content change listener to the organizer manager so that
		 * whenever content is changed, the receiver gets a chance to update.
		 * Likewise, add property change listener to the TaskFilter so that
		 * when it changes, the receiver can update.
		 */
		private void hookContentListeners() {			
			OrganizerManager.instance.addContentChangeListener(contentChangeListener);
			TaskFilter.instance.addPropertyChangeListener(propertyChangeListener);
		}

		private void unhookContentListeners() {
			OrganizerManager.instance.removeContentChangeListener(contentChangeListener);
			TaskFilter.instance.removePropertyChangeListener(propertyChangeListener);
		}		
		
		public Object[] getElements(Object parent) {
			unhookChangeListener();
			
			/*
			 * Spawn  a job to find the tasks. 
			 */
			Job job = new Job("Find Tasks Job") {
				@Override
				protected IStatus run(IProgressMonitor monitor) {
					try {
						fetchTasks();
					} catch (BackendException e) {
						Activator.logError(e);
					}
					return Status.OK_STATUS;
				}
				
			};
			job.schedule();
			
			return new Object[0];
		}
		
		/**
		 * Hook a property change listener to the given task. Should the
		 * task change, the view will be updated.
		 */
		private void hookPropertyChangeListener(Task task) {
			task.addPropertyChangeListener(propertyChangeListener);
		}
		
		private void addTaskToView(final Task task) {
			/*
			 * Spawn a job to do the update; this will ensure that
			 * the workbench is open, and that the update operation
			 * occurs in the UI thread.
			 */
			WorkbenchJob job = new WorkbenchJob("Updating the view") {
				@Override
				public IStatus runInUIThread(IProgressMonitor monitor) {
					viewer.add(task);
					return Status.OK_STATUS;
				}				
			};
			job.schedule();
		}
		
		/**
		 * Stop listening to property change events on all the tasks
		 * we've encountered so far.
		 */
		private void unhookChangeListener() {
			if (tasks == null) return;
			for (Task task : tasks) {
				task.removePropertyChangeListener(propertyChangeListener);
			}
		}

		private void fetchTasks() throws BackendException {
			OrganizerManager.instance.find(TaskFilter.instance, new Visitor() {
				public void visit(Object object) {
					Task task = (Task)object;
					hookPropertyChangeListener(task);
					tasks.add(task);
					addTaskToView(task);
				}						
			});
		}
	}

	public void createPartControl(Composite parent) {
		viewer = new SuperTableViewer(parent, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		new TaskDropTarget(viewer.getTable());
		
		viewer.setContentProvider(new ViewContentProvider());
		getSite().setSelectionProvider(viewer);
		
		makeActions();
		
		hookContextMenu();
		hookDoubleClickAction();
		contributeToActionBars();
		
		// Restore the state of the table; this will create the columns.
		restoreState();
				
		hookListeners();
		viewer.setInput(getViewSite());
	}
	
	@Override
	public void saveState(IMemento memento) {
		// Save the column state.
		IMemento columnsMemento = memento.createChild("columns");
		for (TableViewerColumnDefinition column : viewer.getColumnDefinitions()) {
			IMemento columnMemento = columnsMemento.createChild("column");
			columnMemento.putString("id", column.getId());
			columnMemento.putInteger("width", column.getWidth());
			columnMemento.putString("visible", String.valueOf(column.isVisible()));
		}
		// Save the sort state.
		IMemento sortMemento = memento.createChild("sort");
		sortMemento.putString("column", viewer.getSortColumnId());
		sortMemento.putString("direction", viewer.getSortDirection() == SWT.UP ? "UP" : "DOWN");
	}
	
	@Override
	public void init(IViewSite site, IMemento memento) throws PartInitException {
		this.memento = memento;
		super.init(site, memento);		
	}

	private void hookListeners() {
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				updateActions();
			}
		});
	}
	
	private void updateActions() {
		IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
		boolean isAnythingSelected = selection.size() > 0;
		deleteAction.setEnabled(isAnythingSelected);
		openAction.setEnabled(isAnythingSelected);
//		viewer.update(selection, null);
	}

	private void importTasks(final ArrayList<Task> tasks) {
		ImportTasksWizard.importTasks(getSite().getShell(), tasks);
	}
	
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void fillContextMenu(IMenuManager manager) {
		manager.add(openAction);
		manager.add(createAction);
		manager.add(deleteAction);
		manager.add(refreshAction);
		// Create and populate a submenu that the user can use
		// to show/hide columns.
		IMenuManager columnsMenu = new MenuManager("Columns");
		// TODO Fill the columns menu
		viewer.fillColumnsMenu(columnsMenu);
		manager.add(columnsMenu);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void makeActions() {
		makeOpenAction();
		makeDeleteAction();
		makeCreateAction();
		makeRefreshAction();
		
		updateActions();
	}

	private void makeOpenAction() {
		openAction = new Action() {	
			@Override
			public void run() {
				Task task = (Task)((IStructuredSelection)viewer.getSelection()).getFirstElement();
				IWorkbenchPage page = getSite().getPage(); 
				try {
					page.openEditor(new TaskEditorInput(task), "org.eclipse.organizer.task.ui.editors.TaskEditor");
				} catch (PartInitException e) {
					// TODO Message to user? Log this at least.
					e.printStackTrace();
				}
			}
		};
		openAction.setText("Open");
		openAction.setToolTipText("Open/edit this task");
	}

	private void makeDeleteAction() {
		deleteAction = new Action() {
			@SuppressWarnings("unchecked")
			public void run() {
				ISelection selection = viewer.getSelection();
				List<Task> tasks = (List<Task>)((IStructuredSelection)selection).toList();
				if (tasks == null) return;
				if (tasks.size() == 0) return;
				OrganizerManager.instance.delete(tasks);
		        viewer.refresh(false);
			}	        
		};
		deleteAction.setText("Delete");
		deleteAction.setToolTipText("Delete the currently selected task(s)");
		deleteAction.setImageDescriptor(getSharedImage(ISharedImages.IMG_TOOL_DELETE));
		deleteAction.setDisabledImageDescriptor(getSharedImage(ISharedImages.IMG_TOOL_DELETE_DISABLED));
		//deleteAction.setActionDefinitionId(ActionFactory.DELETE.getId());
	}

	private void makeCreateAction() {
		createAction = new Action() {
			public void run() {
				ArrayList<Task> list = new ArrayList<Task>();
				Task newTask = new Task();
				newTask.setSubject("New Task");
				list.add(newTask);
				importTasks(list);
			}
		};
		createAction.setText("New task...");
		createAction.setToolTipText("Creates a new task.");
		createAction.setImageDescriptor(Activator.getDefault().getImageDescriptor(Activator.IMAGE_NEW_TASK));
		createAction.setDisabledImageDescriptor(getSharedImage(ISharedImages.IMG_TOOL_NEW_WIZARD_DISABLED));
	}
	
	private void makeRefreshAction() {
		refreshAction = new Action() {
			public void run() {
				viewer.refresh();
			}
		};
		refreshAction.setText("Refresh");
		refreshAction.setToolTipText("Refreshes the task list.");
		refreshAction.setImageDescriptor(Activator.getDefault().getImageDescriptor(Activator.IMAGE_REFRESH));
	}
	
	private ImageDescriptor getSharedImage(String id) {
		return PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(id);
	}
	
	private void restoreState() {
		restoreColumnState();
		restoreSortState();
	}
	
	/**
	 * This method makes the columns. It consults with the memento
	 * (provided by a prior call of the init() method) and builds the columns identified
	 * therein. Finally, it builds all the columns that had not been represented in
	 * the memento. Once the columns are built, the sort column and direction are
	 * restored. We do all this to handle the case where some future version of
	 * this class adds (or even removes) columns. 
	 */
	private void restoreColumnState() {
		/*
		 * Build a list of the ids of all the columns that we want to build. As columns
		 * are built, their id is removed from this list so that we know--after the
		 * columns indicated in the memento are built--what other columns remain.
		 */
		List<String> idsOfColumnsThatNeedToBeBuilt = new ArrayList<String>();
		idsOfColumnsThatNeedToBeBuilt.addAll(Arrays.asList(new String[] { "task", "priority", "due", "remind", "subject", "complete" }));
		if (memento != null) {
			IMemento columnsMemento = memento.getChild("columns");
			if (columnsMemento != null) {
				IMemento columnMementos[] = columnsMemento.getChildren("column");
				for (IMemento columnMemento : columnMementos) {
					/*
					 * For each column mentioned in the memento, get the
					 * stored values (id, width, visibility). Use this
					 * information to rebuild the column.
					 */
					String id = columnMemento.getString("id");
					if (id == null) continue;
					int width = Integer.valueOf(columnMemento.getInteger("width"));
					boolean visible = Boolean.valueOf(columnMemento.getString("visible"));
					makeColumn(id, width, visible);
					
					// Remove this column from the list of columns that need to be built.
					idsOfColumnsThatNeedToBeBuilt.remove(id);
				}
			}
		}
		
		// Build any columns that we did not find in the memento.
		for (String id : idsOfColumnsThatNeedToBeBuilt) {
			makeColumn(id, null, null);
		}
		
		restoreSortState();
	
		memento = null;
	}

	/**
	 * This method restores the sort state of the table based on the
	 * information stored in the memento.
	 */
	private void restoreSortState() {
		String sortColumn = "due";
		int sortDirection = SWT.UP;
		
		/*
		 * Get the sort information from the memento if it is available.
		 */
		if (memento != null) {
			IMemento sortMemento = memento.getChild("sort");
			if (sortMemento != null) {
				sortColumn = sortMemento.getString("column");
				sortDirection = "UP".equals(sortMemento.getString("direction")) ? SWT.UP : SWT.DOWN;
			}
		}
		
		if (sortColumn != null) {
			viewer.setSortColumn(sortColumn, sortDirection);
		}
	}
	
	private void makeColumn(String id, Integer width, Boolean initiallyVisible) {
		if ("task".equals(id)) makeTaskColumn(width, initiallyVisible);
		else if ("priority".equals(id)) makePriorityColumn(width, initiallyVisible);
		else if ("due".equals(id)) makeDueDateColumn(width, initiallyVisible);
		else if ("remind".equals(id)) makeReminderDateColumn(width, initiallyVisible);
		else if ("subject".equals(id)) makeSubjectColumn(width, initiallyVisible);
		else if ("complete".equals(id)) makeCompleteColumn(width, initiallyVisible);
		
		// TODO What if we're asked to build a column that doesn't exist?
	}
	
	
	/**
	 * The taskCellDrawListener is added to all the columns displayed in the
	 * table viewer. This listener draws a line through completed tasks.
	 */
	private CellDrawListener taskCellDrawListener = new CellDrawListener() {
		public final void drawCell(CellDrawEvent event) {
			Task task = (Task)event.getObject();
			if (task.isComplete()) {
				GC gc = event.getGC();
				Rectangle bounds = event.getCellBounds();
				
				gc.setForeground(getLineColour(gc.getDevice(), event.isSelected()));				
				int middle = bounds.y + (bounds.height / 2);
				gc.drawLine(bounds.x, middle, bounds.x + bounds.width, middle);
			}
		}

		private Color getLineColour(Device device, boolean isSelected) {
			return device.getSystemColor(isSelected ? SWT.COLOR_LIST_SELECTION_TEXT : SWT.COLOR_DARK_GRAY);
		}
	};

	abstract class AbstractCellLabelProvider extends ColumnLabelProvider {
		@Override
		public Color getForeground(Object element) {
			return getTextColour(viewer.getTable().getDisplay(), (Task)element);
		}

		private Color getTextColour(Device device, Task task) {
			return device.getSystemColor(getTextColourConstant(device, task));
		}
		
		private int getTextColourConstant(Device device, Task task) {
			if (task.isComplete()) return SWT.COLOR_DARK_GRAY;
			if (isPast(task.getDueDate())) return SWT.COLOR_RED;
			return SWT.COLOR_BLACK;
		}
	}
	
	private void makeTaskColumn(Integer savedWidth, Boolean initiallyVisible) {
		final Image image = Activator.getDefault().getImage(Activator.IMAGE_TASK);
		final Rectangle imageBounds = image.getBounds();
		
		TableViewerColumnDefinition taskColumn = new TableViewerColumnDefinition(viewer, SWT.NONE, "task");
		taskColumn.setResizable(false);
		int width = imageBounds.width * 3 / 2; // bug in Windows XP https://bugs.eclipse.org/bugs/show_bug.cgi?id=180783
		taskColumn.setMinimumWidth(width);
		taskColumn.setWidth(width);
		taskColumn.addCellDrawListener(new CellDrawListener() { 			
			public void drawCell(CellDrawEvent event) {
				Rectangle bounds = event.getCellBounds();
				Rectangle imageBounds = image.getBounds();
				event.getGC().drawImage(image, bounds.x + (bounds.width - imageBounds.width) / 2, bounds.y + (bounds.height - imageBounds.width) / 2);
			}
		});
		taskColumn.addCellMeasureListener(new CellMeasureListener() {
			public void measureCell(CellMeasureEvent event) {
				event.setHeight(imageBounds.height + 2);				
			}
			
		});
		//taskColumn.addCellDrawListener(taskCellDrawListener);
		taskColumn.setVisible(true);
	}
		
	private void makeDueDateColumn(Integer savedWidth, Boolean initiallyVisible) {
		dueDateColumn = new TableViewerColumnDefinition(viewer, SWT.RIGHT, "due");
		dueDateColumn.setTitle("Due Date");
		dueDateColumn.setActionText("Due Date");
		dueDateColumn.setActionDescription("Toggles the Due Date column");
		dueDateColumn.setWidth(savedWidth == null ? 120 : savedWidth.intValue());

		final DateFormat format = DateFormat.getDateInstance(DateFormat.LONG);
		
		dueDateColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				Task task = (Task)element;
				Date dueDate = task.getDueDate();
				if (dueDate != null)
					return format.format(dueDate);
				return "";
			}			
		});
		
		dueDateColumn.setEditingSupport(new EditingSupport(viewer) {
			@Override
			protected boolean canEdit(Object element) {
				return true;
			}

			@Override
			protected CellEditor getCellEditor(Object element) {
				return new DateCellEditor(viewer.getTable(), DateFormat.LONG);
			}

			@Override
			protected Object getValue(Object element) {
				return ((Task)element).getDueDate();
			}

			@Override
			protected void setValue(Object element, Object value) {
				((Task)element).setDueDate((Date)value);
				getViewer().update(element, null);
			}			
		});

		dueDateColumn.addCellDrawListener(taskCellDrawListener);
		dueDateColumn.setComparator(new Comparator<Task>() {
			public int compare(Task task0, Task task1) {
				if (task0.getDueDate() == null) {
					if (task1.getDueDate() == null) return 0;
					return -1;
				} else if (task1.getDueDate() == null) return 1;
				else return task0.getDueDate().compareTo(task1.getDueDate());
			}			
		});
		dueDateColumn.setVisible(initiallyVisible == null ? true : initiallyVisible.booleanValue());
	}
	
	private void makeSubjectColumn(Integer savedWidth, Boolean initiallyVisible) {
		subjectColumn = new TableViewerColumnDefinition(viewer, SWT.LEFT, "subject");
		subjectColumn.setTitle("Subject");
		subjectColumn.setActionText("Subject");
		subjectColumn.setActionDescription("Toggles the Subject column.");
		subjectColumn.setWidth(savedWidth == null ? 250 : savedWidth.intValue());
		subjectColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				Task task = (Task)element;
				return task.getSubject();
			}
		});
		subjectColumn.setEditingSupport(new EditingSupport(viewer) {
			@Override
			protected boolean canEdit(Object element) {
				return true;
			}

			@Override
			protected CellEditor getCellEditor(Object element) {
				return new TextCellEditor(viewer.getTable());
			}

			@Override
			protected Object getValue(Object element) {
				return ((Task)element).getSubject();				
			}

			@Override
			protected void setValue(Object element, Object value) {
				((Task)element).setSubject((String)value);
				getViewer().update(element, null);
			}
			
		});
		subjectColumn.addCellDrawListener(taskCellDrawListener);

		subjectColumn.setComparator(new Comparator<Task>() {
			public int compare(Task task0, Task task1) {
				if (task0.getSubject() == null) {
					if (task1.getSubject() == null) return 0;
					return -1;
				} else if (task1.getSubject() == null) return 1;
				else return task0.getSubject().compareToIgnoreCase(task1.getSubject());
			}			
		});
		subjectColumn.setVisible(initiallyVisible == null ? true : initiallyVisible.booleanValue());
	}

	private void makeCompleteColumn(Integer savedWidth, Boolean initiallyVisible) {
		final Image checked = Activator.getDefault().getImage(Activator.IMAGE_CHECKED);
		final Image unchecked = Activator.getDefault().getImage(Activator.IMAGE_UNCHECKED);
		
		completeColumn = new TableViewerColumnDefinition(viewer, SWT.CENTER, "complete");
		completeColumn.setImage(Activator.getDefault().getImage(Activator.IMAGE_CHECKED_HEADER));
		completeColumn.setActionText("Complete");
		completeColumn.setActionDescription("Toggles the Complete column");
		int width = checked.getBounds().width * 3;
		completeColumn.setMinimumWidth(width);
		completeColumn.setWidth(width);
		completeColumn.setResizable(false);
			
		completeColumn.setEditingSupport(new EditingSupport(viewer) {
			@Override
			protected boolean canEdit(Object element) {
				return true;
			}

			@Override
			protected CellEditor getCellEditor(Object element) {
				return new CheckboxCellEditor(viewer.getTable());
			}

			@Override
			protected Object getValue(Object element) {
				return ((Task)element).isComplete();
			}

			@Override
			protected void setValue(Object element, Object value) {
				((Task)element).setComplete((Boolean)value);
			}
			
		});
		
		completeColumn.addCellDrawListener(new CellDrawListener() { 			
			public void drawCell(CellDrawEvent event) {
				Image image = getImage((Task)event.getObject());
				Rectangle imageBounds = image.getBounds();
				Rectangle bounds = event.getCellBounds();
				event.getGC().drawImage(image, bounds.x + (bounds.width - imageBounds.width) / 2, bounds.y + (bounds.height - imageBounds.width) / 2);
			}
	
			private Image getImage(Task task) {
				return task.isComplete() ? checked : unchecked;
			}			
		});
		completeColumn.setComparator(new Comparator<Task>() {
			public int compare(Task task0, Task task1) {
				return Boolean.valueOf(task0.isComplete()).compareTo(Boolean.valueOf(task1.isComplete()));
			}			
		});
		completeColumn.setVisible(initiallyVisible == null ? true : initiallyVisible.booleanValue());
	}

	private void makeReminderDateColumn(Integer savedWidth, Boolean initiallyVisible) {
		reminderDateColumn = new TableViewerColumnDefinition(viewer, SWT.RIGHT, "remind");
		reminderDateColumn.setTitle("Reminder Date");
		reminderDateColumn.setActionText("Reminder Date");
		reminderDateColumn.setActionDescription("Toggles the Reminder Date column");
		reminderDateColumn.setWidth(savedWidth == null ? 100 : savedWidth.intValue());
		reminderDateColumn.setEditingSupport(new EditingSupport(viewer) {
			@Override
			protected boolean canEdit(Object element) {
				return true;
			}

			@Override
			protected CellEditor getCellEditor(Object element) {
				return new DateCellEditor(viewer.getTable(), DateFormat.LONG);
			}

			@Override
			protected Object getValue(Object element) {
				return ((Task)element).getReminderDate();
			}

			@Override
			protected void setValue(Object element, Object value) {
				((Task)element).setReminderDate((Date)value);
				getViewer().update(element, null);				
			}
			
		});

		final DateFormat format = DateFormat.getDateInstance(DateFormat.LONG);
		
		reminderDateColumn.setLabelProvider(new AbstractCellLabelProvider() {
			@Override
			public String getText(Object element) {
				Task task = (Task)element;
				Date reminderDate = task.getReminderDate();
				if (reminderDate != null)
					return format.format(reminderDate);
				return "";
			}			
		});
		
		reminderDateColumn.addCellDrawListener(taskCellDrawListener);
		reminderDateColumn.setComparator(new Comparator<Task>() {
			public int compare(Task task0, Task task1) {
				if (task0.getReminderDate() == null) {
					if (task1.getReminderDate() == null) return 0;
					return -1;
				} else if (task1.getReminderDate() == null) return 1;
				else return task0.getReminderDate().compareTo(task1.getReminderDate());
			}			
		});
		
		reminderDateColumn.setVisible(initiallyVisible == null ? false : initiallyVisible.booleanValue());
	}
	
	private void makePriorityColumn(Integer savedWidth, Boolean initiallyVisible) {
		
		final Image low = Activator.getDefault().getImage(Activator.IMAGE_LOW);
		final Image normal = Activator.getDefault().getImage(Activator.IMAGE_NORMAL);
		final Image high = Activator.getDefault().getImage(Activator.IMAGE_HIGH);
	
		priorityColumn = new TableViewerColumnDefinition(viewer, SWT.CENTER, "priority");
		priorityColumn.setTitle("!");
		priorityColumn.setActionText("Priority");
		priorityColumn.setActionDescription("Toggles the Priority column");
		priorityColumn.setWidth(low.getBounds().width * 2);
		priorityColumn.setMinimumWidth(low.getBounds().width * 2);
		priorityColumn.setResizable(false);
		
		priorityColumn.addCellDrawListener(taskCellDrawListener);
		
		priorityColumn.setEditingSupport(new EditingSupport(viewer) {
			private String[] priorities = new String[] {"Low", "Normal", "High"};

			@Override
			protected boolean canEdit(Object element) {
				return true;
			}

			@Override
			protected CellEditor getCellEditor(Object element) {
				return new ComboBoxCellEditor(viewer.getTable(), priorities);
			}

			@Override
			protected Object getValue(Object element) {
				Priority priority = ((Task)element).getPriority();
				if (priority == Priority.LOW) return 0;
				else if (priority == Priority.NORMAL) return 1;
				else if (priority == Priority.HIGH) return 2;
				else return null;
			}

			@Override
			protected void setValue(Object element, Object value) {
				Priority priority = null;
				switch (((Integer)value).intValue()) {
				case 0 : priority = Priority.LOW; break;
				case 1 : priority = Priority.NORMAL; break;
				case 2 : priority = Priority.HIGH; break;
				}
				((Task)element).setPriority(priority);
			}
		});
	
		priorityColumn.addCellDrawListener(new CellDrawListener() { 			
			public void drawCell(CellDrawEvent event) {
				Task task = (Task)event.getObject();
				Image image = getImage(task.getPriority());
				Rectangle imageBounds = image.getBounds();
				Rectangle bounds = event.getCellBounds();
				event.getGC().drawImage(image, bounds.x + (bounds.width - imageBounds.width) / 2, bounds.y + (bounds.height - imageBounds.width) / 2);
			}
	
			public Image getImage(Priority priority) {
				if (priority == Priority.LOW) return low;
				else if (priority == Priority.NORMAL) return normal;
				else if (priority == Priority.HIGH) return high;
				else return null;
			}		
		});
		
		priorityColumn.setComparator(new Comparator<Task>() {
			public int compare(Task task0, Task task1) {
				return task0.getPriority().compareTo(task1.getPriority());
			}			
		});
		
		priorityColumn.setVisible(initiallyVisible == null ? false : initiallyVisible.booleanValue());
	}
	
	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (openAction.isEnabled()) openAction.run();
			}
		});
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalToolBar(bars.getToolBarManager());
	}
	
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(refreshAction);
		manager.add(createAction);
		manager.add(deleteAction);
	}
	
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	private boolean isPast(Date date) {
		if (date == null) return false;
		return date.compareTo(new Date()) < 0;
	}

}
