/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * This class is derived from code contributed by Bahadir Yagan in
 * bug 19945 (https://bugs.eclipse.org/bugs/show_bug.cgi?id=19945#c11), 
 * though this implementation is almost a complete rewrite (little of the 
 * original code remains).
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/

package org.eclipse.samples.organizer.ui.datepicker;

import java.text.ParseException;
import java.util.Date;
import java.util.Locale;

import org.eclipse.core.runtime.ListenerList;
import org.eclipse.nebula.widgets.cdatetime.CDT;
import org.eclipse.nebula.widgets.cdatetime.CDateTime;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.HelpListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.ibm.icu.text.DateFormat;
import com.ibm.icu.util.ULocale;

public class DatePickerCombo extends Composite {
	private Text dateText = null;
	private Button pickButton = null;
	private Date date = null;
	private DateFormat format;
	private ULocale locale;

	private ListenerList dateChangeListeners = new ListenerList();
	
	public DatePickerCombo(Composite parent, int style) {
		this(parent, style, ULocale.getDefault());
	}

	public DatePickerCombo(Composite parent, int style, Locale locale) {
		this(parent, style, ULocale.forLocale(locale));
	}
	
	public DatePickerCombo(Composite parent, int style, ULocale locale) {
		super(parent, style);
		this.locale = locale;
		format = DateFormat.getDateTimeInstance(DateFormat.FULL, -1, locale);
		initialize(style);
	}

	private void initialize(int style) {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;
		gridLayout.makeColumnsEqualWidth = false;		
		this.setLayout(gridLayout);
				
		dateText = new Text(this, SWT.NONE);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = SWT.FILL;
		dateText.setLayoutData(gridData);
		dateText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent e) {
				if (e.keyCode == SWT.ARROW_DOWN) showDateDropdown();
			}			
		});
		dateText.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent event) {
				try {
					internalSetDate(format.parse(dateText.getText()));
				} catch (ParseException e) {
					// If we can't parse the text into a date, return to
					// the previous value.
					updateDateText();
				}
			}
		});		
		
		TraverseListener traverseListener = new TraverseListener() {
			public void keyTraversed(TraverseEvent event) {
				if (event.keyCode == SWT.TAB || event.keyCode == SWT.CR) {
					pickButton.getParent().traverse(SWT.TRAVERSE_TAB_NEXT);
				} else if (event.keyCode == SWT.ARROW_DOWN) {
					showDateDropdown();
				}
			}
		};
		dateText.addTraverseListener(traverseListener);
		
		pickButton = new Button(this, SWT.ARROW | SWT.DOWN | SWT.FLAT);
		gridData = new GridData();
		gridData.horizontalAlignment = SWT.END;
		gridData.verticalAlignment = SWT.FILL;
		
		pickButton.setLayoutData(gridData);
		pickButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				showDateDropdown();
			}
		});
		pickButton.addTraverseListener(traverseListener);
		setTabList(new Control[] {dateText});
	}

	public Date getDate() {
		return date;
	}

	public void setDate(Date date) {
		this.date = date;
		updateDateText();
	}
	
	private void internalSetDate(Date date) {
		Date oldDate = this.date;
		setDate(date);
		fireDateChangedEvent(oldDate, date);
	}
	
	@Override
	public void addListener(int eventType, Listener listener) {
		dateText.addListener(eventType, listener);
	}
	
	private void showDateDropdown() {
		final Shell shell = new Shell(this.getShell(), SWT.ON_TOP);
		shell.setText("Shell"); //$NON-NLS-1$
		GridLayout gridLayout = new GridLayout();
		shell.setLayout(gridLayout);
		
		final CDateTime datePicker = new CDateTime(shell, CDT.SIMPLE | CDT.COMPACT);
		
//		DatePicker datePicker = new DatePicker(shell, SWT.NONE, locale);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.verticalAlignment = SWT.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.grabExcessHorizontalSpace = true;
		datePicker.setLayoutData(gridData);
		
		datePicker.setSelection(date);
		
		datePicker.addSelectionListener(new SelectionAdapter() {
			public void widgetDefaultSelected(SelectionEvent e) {
				Date date = (Date)e.data;
				shell.close();
				internalSetDate(date);
			}
		});
				
		Composite buttons = createButtonsPanel(shell, datePicker);
		gridData = new GridData();
		gridData.horizontalAlignment = SWT.CENTER;
		buttons.setLayoutData(gridData);

		shell.pack(); 

		Rectangle displayBounds = shell.getDisplay().getBounds();
		Rectangle pickButtonBounds = pickButton.getBounds();
		Rectangle shellBounds = shell.getBounds();
		
		int left = pickButtonBounds.x + pickButtonBounds.width - shellBounds.width;
		int top = pickButtonBounds.y + pickButtonBounds.height;

		Point where = toDisplay(new Point(left, top));
		
		int maxX = displayBounds.width - shellBounds.width;
		int maxY = displayBounds.height - shellBounds.height;
		
		if (where.x < 0) where.x = 0;
		if (where.y < 0) where.y = 0;
		if (where.x > maxX) where.x = maxX;
		if (where.y > maxY) where.y = maxY;
		
		shell.setLocation(where);
		shell.open();
		datePicker.setFocus();
		
		shell.addShellListener(new ShellAdapter() {
			public void shellDeactivated(ShellEvent e) {
				shell.close();
			}
		});
		
		shell.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if (e.keyCode == SWT.ESC) shell.close();
			}
		});
	}

	private Composite createButtonsPanel(Shell shell, final CDateTime datePicker) {
		Composite buttons = new Composite(shell, SWT.NONE);					
		RowLayout rowLayout = new RowLayout();
		rowLayout.justify = true;
		buttons.setLayout(rowLayout);
		
		Button todayButton = new Button(buttons, SWT.PUSH);
		todayButton.setText(Messages.getString("today")); //$NON-NLS-1$
		todayButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				datePicker.getShell().close();
				internalSetDate(new Date());
			}			
		});
		
		Button noneButton = new Button(buttons, SWT.PUSH);
		noneButton.setText(Messages.getString("none")); //$NON-NLS-1$
		noneButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				datePicker.getShell().close();
				internalSetDate(null);
			}			
		});
		
		noneButton.addTraverseListener(new TraverseListener() {
			public void keyTraversed(TraverseEvent e) {
				switch (e.detail) {
				case SWT.TRAVERSE_TAB_NEXT:
					datePicker.setFocus();
					e.doit = false;
					break;
				}
			}
		});
		
		return buttons;
	}

	private void updateDateText() {
		dateText.setText(date == null ? "" : format.format(date)); //$NON-NLS-1$
	}
	
	public void addDateSelectionListener(DateSelectionListener listener) {
		dateChangeListeners.add(listener);
	}

	public void removeDateSelectionListener(DateSelectionListener listener) {
		dateChangeListeners.remove(listener);
	}
	
	private void fireDateChangedEvent(Date oldDate, Date newDate) {
		DateSelectedEvent event = new DateSelectedEvent(this, oldDate, newDate);
		for (Object object : dateChangeListeners.getListeners()) {
			DateSelectionListener listener = (DateSelectionListener)object;
			listener.dateSelected(event);
		}
	}

	public void setDateFormat(int style) {
		this.format = DateFormat.getDateInstance(style, locale);
	}
	
	@Override
	public boolean setFocus() {
		return pickButton.setFocus();
	}
	
	@Override
	public void setData(String key, Object value) {
		super.setData(key, value);
		dateText.setData(key, value);
		pickButton.setData(key, value);
	}
	
	@Override
	public void addHelpListener(HelpListener listener) {
		super.addHelpListener(listener);
		dateText.addHelpListener(listener);
		pickButton.addHelpListener(listener);
	}
	
	@Override
	public void removeHelpListener(HelpListener listener) {
		super.removeHelpListener(listener);
		dateText.removeHelpListener(listener);
		pickButton.removeHelpListener(listener);
	}
} 