/*******************************************************************************
 * Copyright (c) 2006, 2007 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TableColumn;

/**
 * The SuperTableViewer provides some features over and above what the standard
 * TableViewer provides. In particular, instances of this class provide:
 * <ul>
 * <li>a paint listener to customize the appearance of table cells</li>
 * <li>columns that can be hidden and shown<li>
 * <li>automatic sorting (both ascending and descending) based on column selection<li>
 * </ul>
 * Only instances of {@link TableViewerColumnDefinition} should be used to build columns
 * for instances of this class. 
 */
public class SuperTableViewer extends TableViewer {

	private List<TableViewerColumnDefinition> columnDefinitions = new ArrayList<TableViewerColumnDefinition>();

	/**
	 * The paintListener is invoked <em>after</em> a cell has already
	 * been drawn. This provides an opportunity to draw on top of what is
	 * shown by default.
	 */
	private Listener paintListener = new Listener() {
		public void handleEvent(Event event) {
			CellDrawEvent cellDrawEvent = new CellDrawEvent(event);
			cellDrawEvent.getColumnDefinition().drawCell(cellDrawEvent);
		}
	};
	
	private Listener measureListener = new Listener() {
		public void handleEvent(Event event) {
			CellMeasureEvent cellMeasureEvent = new CellMeasureEvent(event);
			cellMeasureEvent.getColumnDefinition().measureCell(cellMeasureEvent);
		}
	};
	
	public SuperTableViewer(Composite parent, int style) {
		super(parent, style);
		hookEventListeners();
		getTable().addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				unhookListeners();
			}
		});
	}

	protected void addColumn(TableViewerColumnDefinition column) {
		columnDefinitions.add(column);
	}

	private void hookEventListeners() {
		getTable().addListener(SWT.PaintItem, paintListener);
		getTable().addListener(SWT.MeasureItem, measureListener);
	}

	private void unhookListeners() {
		getTable().removeListener(SWT.PaintItem, paintListener);
		getTable().removeListener(SWT.MeasureItem, measureListener);
	}

	/**
	 * This method fills the given menu manager with an action for each column
	 * that has one. The action is used to hide and show the column.
	 * 
	 * @param manager
	 *            The instance of {@link IMenuManager} to populate with column
	 *            actions.
	 */
	public void fillColumnsMenu(IMenuManager manager) {
		for (TableViewerColumnDefinition column : columnDefinitions) {
			if (column.hasAction())
				manager.add(column.getAction());
		}
	}
	
	public List<TableViewerColumnDefinition> getColumnDefinitions() {
		return columnDefinitions;
	}
	
	public int getColumnIndex(TableViewerColumnDefinition target) {
		int index = 0;
		for (TableViewerColumnDefinition column : columnDefinitions) {
			if (column == target) return index;
			if (column.isVisible()) index++;
		}
		// This can only happen if we don't know about the column (which is a little scary).
		// TODO Should this throw a runtime exception?
		return columnDefinitions.size() - 1; 
	}

	public void columnMoved(TableViewerColumnDefinition tableViewerColumnDefinition) {
		TableViewerColumnDefinition[] visibleColumns = getVisibleColumns();
		int count = 0;
		for (int index=0;index<columnDefinitions.size();index++) {
			TableViewerColumnDefinition column = columnDefinitions.get(index);
			if (column.isVisible()) {
				columnDefinitions.set(index, visibleColumns[count++]);
			}
		}
	}

	private TableViewerColumnDefinition[] getVisibleColumns() {
		TableViewerColumnDefinition[] visibleColumns = new TableViewerColumnDefinition[getTable().getColumnCount()];
		TableColumn[] columns = getTable().getColumns();
		for(int index=0;index<columns.length;index++) {
			TableColumn column = columns[index];
			int position = getCurrentColumnIndex(column);
			visibleColumns[position] = getTableViewerColumnDefinitionForTableColumn(column);
		}
		return visibleColumns;
	}

	private TableViewerColumnDefinition getTableViewerColumnDefinitionForTableColumn(TableColumn column) {
		if (column == null) return null;
		return (TableViewerColumnDefinition) column.getData(TableViewerColumnDefinition.SUPER_COLUMN_DATA_INDEX);
	}

	private int getCurrentColumnIndex(TableColumn column) {
		int initialIndex = getInitialColumnIndex(column);
		int[] indices = getTable().getColumnOrder();
		for(int index = 0;index<indices.length;index++) {
			if (indices[index] == initialIndex) return index;
		}
		return 0;
	}

	private int getInitialColumnIndex(TableColumn column) {
		TableColumn[] columns = getTable().getColumns();
		for (int index=0;index<columns.length;index++) {
			if (columns[index]==column) return index;
		}
		// TODO Throw a runtime exception instead?
		return columns.length - 1;
	}

	public TableViewerColumnDefinition getSortColumn() {
		return getTableViewerColumnDefinitionForTableColumn(getTable().getSortColumn());
	}

	public int getSortDirection() {
		return getTable().getSortDirection();
	}

	public void setSortColumn(String id, int sortDirection) {
		TableViewerColumnDefinition column = getColumnDefinition(id);
		if (column == null) return;
		column.setSortColumn(sortDirection);
	}

	private TableViewerColumnDefinition getColumnDefinition(String id) {
		for (TableViewerColumnDefinition column : columnDefinitions) {
			if (column.getId().equals(id)) return column;
		}
		return null;
	}

	public String getSortColumnId() {
		TableViewerColumnDefinition sortColumn = getSortColumn();
		if (sortColumn == null) return null;
		return sortColumn.getId();
	}
}
