/*******************************************************************************
 * Copyright (c) 2006, 2007 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.util;

import java.util.Comparator;

import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.TableColumn;

public class TableViewerColumnDefinition {

	protected static final String SUPER_COLUMN_DATA_INDEX = "super_column";

	private static final CellLabelProvider defaultLabelProvider = new ColumnLabelProvider() {
		@Override
		public String getText(Object element) {
			return "";
		}
	};

	private final int style;
	private String title;
	private Image image;
	private boolean resizable = true;
	private boolean moveable = true;
	private int minimumWidth = 100;
	private int width = minimumWidth;
	
	@SuppressWarnings("unchecked")
	private Comparator comparator;

	private CellLabelProvider cellLabelProvider = defaultLabelProvider;
	
	private EditingSupport editingSupport;
	
	/**
	 * This field references the {@link TableViewer} for which this
	 * instance will create columns.
	 */
	private final SuperTableViewer viewer;
		
	/**
	 * This field contains a {@link ListenerList} of {@link CellDrawListener}
	 * instances that are notified when an entry is drawn for this instance.
	 */
	private ListenerList cellDrawListenerList = new ListenerList();

	private ListenerList cellMeasureListenerList = new ListenerList();
	
	/**
	 * This field contains the instance of {@link TableViewerColumn} that
	 * is created/managed by this instance. This field only contains a
	 * value after the column has been created/shown.
	 */
	private TableViewerColumn columnViewer;
	
	/**
	 * This field contains the instance of {@link TableColumn} created
	 * by this instance. If this field is null, then the column does
	 * not exist.
	 */
	protected TableColumn column;
	
	/**
	 * The action field contains an {@link IAction} that can be used to
	 * hide and show the column.
	 */
	private IAction action = new Action() {
		@Override
		public void run() {
			toggleVisibility();
		}
	};
	
	/**
	 * When {@link SelectionListener} is notified, it sets the sorter on
	 * the {@link TableViewer}. If a new column is selected, the sort order
	 * is ascending. If the currently selected column is reselected, the sort
	 * order is toggled between ascending and descending. 
	 */
	private SelectionAdapter selectionListener = new SelectionAdapter() {
		@Override
		public void widgetSelected(SelectionEvent e) {
			setSortColumn();
		}
	};
	
	/**
	 * The controlListener is notified whenever the column is moved
	 * or resized. Move events are used to keep track of column order 
	 * (so that columns can be restored to their original position when
	 * they are hidden/shown). Resize events are consumed to maintain
	 * the minimumWidth constraint.
	 */
	private ControlListener controlListener = new ControlListener() {
		public void controlMoved(ControlEvent e) {
			viewer.columnMoved(TableViewerColumnDefinition.this);
		}
		
		public void controlResized(ControlEvent e) {
			width = column.getWidth();
			if (width < minimumWidth) {
				column.setWidth(minimumWidth);
				width = minimumWidth;
			}
		}
	};

	private String id;
	
	public TableViewerColumnDefinition(SuperTableViewer viewer, int style, String id) {
		this.viewer = viewer;
		this.style = style;
		this.id = id;
		
		viewer.addColumn(this);
	}

	public void setResizable(boolean resizable) {
		this.resizable = resizable;		
	}

	public void setMinimumWidth(int minimumWidth) {
		this.minimumWidth = minimumWidth;		
	}

	public void setWidth(int width) {
		this.width = width;		
	}

	public void addCellDrawListener(CellDrawListener cellDrawListener) {
		cellDrawListenerList .add(cellDrawListener);
	}
	
	public void removeCellDrawListener(CellDrawListener cellDrawListener) {
		cellDrawListenerList.remove(cellDrawListener);
	}

	public void addCellMeasureListener(CellMeasureListener cellMeasureListener) {
		cellMeasureListenerList .add(cellMeasureListener);
	}
	
	public void removeCellMeasureListener(CellMeasureListener cellMeasureListener) {
		cellMeasureListenerList.remove(cellMeasureListener);
	}
	
	/**
	 * The title that will be displayed in the column header.
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	public void setActionDescription(String actionDescription) {
		action.setDescription(actionDescription);	
	}

	/**
	 * {@link EditingSupport} that determines how entries in this column are
	 * edited. The column's entries are not editable if this value is not
	 * provided.
	 */
	public void setEditingSupport(EditingSupport editingSupport) {
		this.editingSupport = editingSupport;
	}

	/**
	 * The 'comparator' contains an instance of {@link Comparator}
	 * that is used to sort the table based on this column. This comparator
	 * is used for both ascending and descending sorts (depending on what
	 * order the values are provided). This field is optional, if it is
	 * not provided, the table cannot be sorted on this column.
	 */
	@SuppressWarnings("unchecked")
	public void setComparator(Comparator comparator) {
		this.comparator = comparator;
	}
	
	protected ViewerSorter getDescendingSorter() {
		if (comparator == null) return null;
		return new ViewerSorter() {
			@SuppressWarnings("unchecked")
			@Override
			public int compare(Viewer viewer, Object e1, Object e2) {
				return comparator.compare(e2, e1);
			}
		};
	}

	protected ViewerSorter getAscendingSorter() {
		if (comparator == null) return null;
		return new ViewerSorter() {
			@SuppressWarnings("unchecked")
			@Override
			public int compare(Viewer viewer, Object e1, Object e2) {
				return comparator.compare(e1, e2);
			}
		};
	}
	
	protected void toggleVisibility() {
		setVisible(!isVisible());
		viewer.refresh();
	}

	public void setVisible(boolean value) {
		if (value) {
			createColumn();
		} else {
			disposeColumn();
		}
		action.setChecked(value);
	}
	
	protected void createColumn() {
		if (isVisible()) return;
		columnViewer = new TableViewerColumn(viewer, style, getColumnIndex());
		columnViewer.setLabelProvider(cellLabelProvider);
		columnViewer.setEditingSupport(editingSupport);
		
		column = columnViewer.getColumn();		
		column.setData(SUPER_COLUMN_DATA_INDEX, this);
		column.setResizable(isResizable());
		column.setText(title == null ? "" : title);
		column.setImage(image);
		column.setMoveable(moveable);
		column.setWidth(width < minimumWidth ? minimumWidth : width); 
		hookListeners();
	}

	private int getColumnIndex() {
		return viewer.getColumnIndex(this);
	}

	public boolean isResizable() {
		return resizable;
	}

	public void dispose() {
		disposeColumn();
	}
	
	protected void disposeColumn() {
		if (!isVisible()) return;
		columnViewer = null;
		unhookListeners();
		column.dispose();
		column = null;
	}
	
	protected void hookListeners() {
		column.addSelectionListener(selectionListener);
		column.addControlListener(controlListener);
	}

	protected void unhookListeners() {
		column.removeSelectionListener(selectionListener);
		column.removeControlListener(controlListener);
	}

	public void drawCell(CellDrawEvent event) {
		Object[] listeners = cellDrawListenerList.getListeners();
		for (int index=0;index<listeners.length;index++) {
			CellDrawListener listener = (CellDrawListener)listeners[index];
			listener.drawCell(event);
		} 
	}

	public void measureCell(CellMeasureEvent event) {
		Object[] listeners = cellMeasureListenerList.getListeners();
		for (int index=0;index<listeners.length;index++) {
			CellMeasureListener listener = (CellMeasureListener)listeners[index];
			listener.measureCell(event);
		} 
	}
	
	/**
	 * The LabelProvider is used to provide a label for cells in the
	 * column represented by this instance.
	 */
	public void setLabelProvider(CellLabelProvider cellLabelProvider) {
		this.cellLabelProvider = cellLabelProvider;		
	}

	public IAction getAction() {
		return action;
	}

	public boolean hasAction() {
		return action.getText() != null;
	}

	public boolean isVisible() {
		return columnViewer != null;
	}


	public void setImage(Image image) {
		this.image = image;		
	}

	public void setActionText(String text) {
		this.action.setText(text);
	}

	
	public String getText() {
		return title;
	}

	public int getWidth() {
		return width;
	}

	/**
	 * The id uniquely identifies the receiver within the table.
	 * @return
	 */
	public String getId() {
		return id;
	}
	
	public void setSortColumn(int direction) {
		if (direction == SWT.UP) {
			sortAscending();
		} else {
			sortDescending();
		}
	}
	
	public void setSortColumn() {
		// If there is no sorter, ignore the event.
		if (comparator == null) return;
		
		if (viewer.getTable().getSortColumn() == column) {
			if (viewer.getTable().getSortDirection() == SWT.UP) {
				sortDescending();
			} else {
				sortAscending();
			}
		} else {
			sortDescending();
		}
	}

	private void sortAscending() {
		setSorter(getAscendingSorter(), SWT.UP);
	}
	
	private void sortDescending() {
		setSorter(getDescendingSorter(), SWT.DOWN);
	}
	
	private void setSorter(ViewerSorter sorter, int direction) {
		viewer.getTable().setSortColumn(column);
		viewer.getTable().setSortDirection(direction);	
		viewer.setSorter(sorter);
	}

}
